import os
import pytest

from wulaisdk.request import CommonRequest
from wulaisdk.exceptions import ClientException

pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")


@pytest.mark.parametrize('action,params,opts,expected', [
    ('/user/create',
     {
         "avatar_url": "",
         "user_id": "shierlou",
         "nickname": "测试用户-shierlou"
     },
     {
         "method": "POST",
         "timeout": 3,
         "retry": 0,
     }, CommonRequest),
    ('/user/create',
     {
         "avatar_url": "",
         "user_id": "shierlou",
         "nickname": "测试用户-shierlou"
     },
     {
         "method": "POST",
         "timeout": 3,
         "retry": 3,
     }, CommonRequest),
])
def test_common_request_normal(action, params, opts, expected):
    request = CommonRequest(action, params, opts)
    assert isinstance(request, expected)


@pytest.mark.parametrize('action,params,opts,mode', [
    ('/user/create',
     '{"avatar_url": "", "user_id": "shierlou", "nickname": "测试用户-shierlou"}',
     {
         "method": "GET",
         "timeout": 3,
         "retry": 0,
     }, 1),
    ('/user/create',
     {
         "avatar_url": "",
         "user_id": "shierlou",
         "nickname": "测试用户-shierlou"
     },
     '{"method": "POST", "timeout": 3, "retry": 3}', 1),
    ({"url": "user/create"},
     {
         "avatar_url": "",
         "user_id": "shierlou",
         "nickname": "测试用户-shierlou"
     },
     {
         "method": "GET",
         "timeout": 3,
         "retry": 0,
     }, 2),
])
def test_common_request_error(action, params, opts, mode):
    if mode == 1:
        with pytest.raises(ClientException) as excinfo:
            CommonRequest(action, params, opts)
            assert excinfo.error_msg == "SDK_INVALID_PARAMS"
    elif mode == 2:
        with pytest.raises(ClientException) as excinfo:
            CommonRequest(action, params, opts)
            assert excinfo.error_msg == "SDK_INVALID_ACTION"


@pytest.mark.parametrize('headers,excepted', [
    ({"a": 1}, {
        "a": 1
    }),
])
def test_set_headers(headers, excepted):
    action = '/user/create'
    params = {
     "avatar_url": "",
     "user_id": "shierlou",
     "nickname": "测试用户-shierlou"
    }
    opts = {
     "method": "POST",
     "timeout": 3,
     "retry": 0,
    }
    request = CommonRequest(action, params, opts)
    request.set_headers(headers)
    assert request.headers == excepted


@pytest.mark.parametrize('k,v', [
    ("a", 1),
])
def test_add_headers(k, v):
    action = '/user/create'
    params = {
     "avatar_url": "",
     "user_id": "shierlou",
     "nickname": "测试用户-shierlou"
    }
    opts = {
     "method": "POST",
     "timeout": 3,
     "retry": 0,
    }
    request = CommonRequest(action, params, opts)
    request.add_headers(k, v)
    # now headers
    # {
    #     "Accept": "application/json",
    #     "Content-Type": "application/json",
    #     "a": 1,
    #     "User-Agent": "xxx"
    # }
    assert "a" in request.headers
    assert "Accept" in request.headers
    assert "Content-Type" in request.headers
    assert "User-Agent" in request.headers




import os
import time
import pytest

from wulaisdk.client import WulaiClient
from wulaisdk.exceptions import ClientException


pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")
log_dir_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


#############################################################
#                           统计类
#############################################################
# 添加用户满意度评价
# 步骤: 1. bot-response 2. sync 3. this api
@pytest.mark.parametrize('user_id, satisfaction', [
    ("shierlou", "THUMB_UP"),
    ("shierlou", "BAD_ANSWER"),
    ("shierlou", "WRONG_ANSWER"),
    ("shierlou", "REPORT")
])
def test_create_qa_satisfaction(user_id, satisfaction):
    client = WulaiClient(pubkey, secret, debug=True)

    # 获取机器人回复
    msg_body = {"text": {"content": "你好"}}
    resp_bot_response = client.get_bot_response(user_id, msg_body)
    bot = resp_bot_response.suggested_response[0].bot
    msg_body_response = resp_bot_response.suggested_response[0].response[0].msg_body.to_dict()

    # 同步发给用户的消息（机器人回复）
    msg_ts = str(int(time.time() * 1000))
    resp_sync = client.sync_message(user_id, msg_body_response, msg_ts, bot=bot.to_dict())

    # msg_id
    msg_id = resp_sync.msg_id
    # knowledge_id
    knowledge_id = bot.qa.knowledge_id

    resp = client.create_qa_satisfaction(msg_id, user_id, {"knowledge_id": str(knowledge_id)}, satisfaction)
    assert resp == {}


# 查询问答满意度评价统计列表（知识点粒度，日报）
@pytest.mark.parametrize('start_date, end_date, page, page_size', [
    ("20190930", "20191030", 1, 50),
    ("20190930", "20191030", 2, 20),
])
def test_stats_qa_satisfaction_daily_knowledges_normal(start_date, end_date, page, page_size):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.stats_qa_satisfaction_daily_knowledges(start_date, end_date, page, page_size)
    assert resp.to_dict()


@pytest.mark.parametrize('start_date, end_date, page, page_size', [
    ("20190830", "20191030", 1, 50),
    ("20190930", "20190830", 2, 20),
])
def test_stats_qa_satisfaction_daily_knowledges_error(start_date, end_date, page, page_size):
    client = WulaiClient(pubkey, secret, debug=True)
    with pytest.raises(ClientException) as excinfo:
        client.stats_qa_satisfaction_daily_knowledges(start_date, end_date, page, page_size)
        assert excinfo.error_code == "SDK_INVALID_PARAMS"


# 查询问答召回数统计列表（日报）
@pytest.mark.parametrize('start_date, end_date', [
    ("20190930", "20191030"),
])
def test_stats_qa_recall_daily_normal(start_date, end_date):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.stats_qa_recall_daily(start_date, end_date)
    assert resp.to_dict()


# 查询问答召回数统计列表（知识点粒度，日报）
@pytest.mark.parametrize('start_date, end_date, page, page_size', [
    ("20190930", "20191030", 1, 50),
    ("20190930", "20191030", 2, 20),
])
def test_stats_qa_recall_daily_knowledges_normal(start_date, end_date, page, page_size):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.stats_qa_recall_daily_knowledges(start_date, end_date, page, page_size)
    assert resp.to_dict()



from typing import List

from wulaisdk.response import BaseModel
from wulaisdk.response.bot import Response


class Keyword(BaseModel):
    keyword_id: int
    keyword: str

    def __init__(self, keyword_id: int, keyword: str) -> None:
        self.keyword_id = keyword_id
        self.keyword = keyword


class KeywordSuggestedResponse(BaseModel):
    is_send: bool
    score: int
    response: List[Response]
    keyword: Keyword
    quick_reply: List[str]

    def __init__(self, is_send: bool, score: int, response: List[Response], keyword: Keyword, quick_reply: List[str]) -> None:
        self.is_send = is_send
        self.score = score
        self.response = [Response.from_dict(r) for r in response]
        self.keyword = Keyword.from_dict(keyword)
        self.quick_reply = quick_reply


class KeywordBotResponse(BaseModel):
    is_dispatch: bool
    msg_id: str
    keyword_suggested_response: List[KeywordSuggestedResponse]
    extra: str

    def __init__(self, is_dispatch: bool, msg_id: str, keyword_suggested_response: List[KeywordSuggestedResponse], extra: str) -> None:
        self.is_dispatch = is_dispatch
        self.msg_id = msg_id
        self.keyword_suggested_response = [KeywordSuggestedResponse.from_dict(ksr) for ksr in keyword_suggested_response]
        self.extra = extra



from wulaisdk.response import BaseModel


class Text(BaseModel):
    content: str

    def __init__(self, content: str) -> None:
        self.content = content


class Image(BaseModel):
    resource_url: str

    def __init__(self, resource_url: str) -> None:
        self.resource_url = resource_url


class Custom(BaseModel):
    content: str

    def __init__(self, content: str) -> None:
        self.content = content


class Video(BaseModel):
    resource_url: str
    thumb: str
    description: str
    title: str

    def __init__(self, resource_url: str, thumb: str, description: str, title: str) -> None:
        self.resource_url = resource_url
        self.thumb = thumb
        self.description = description
        self.title = title


class File(BaseModel):
    file_name: str
    resource_url: str

    def __init__(self, file_name: str, resource_url: str) -> None:
        self.file_name = file_name
        self.resource_url = resource_url


class Voice(BaseModel):
    # todo: type
    resource_url: str
    type: str
    recognition: str

    def __init__(self, resource_url: str, type: str, recognition: str) -> None:
        self.resource_url = resource_url
        self.type = type
        self.recognition = recognition


class ShareLink(BaseModel):
    description: str
    destination_url: str
    cover_url: str
    title: str

    def __init__(self, description: str, destination_url: str, cover_url: str, title: str) -> None:
        self.description = description
        self.destination_url = destination_url
        self.cover_url = cover_url
        self.title = title


class MsgBody(BaseModel):

    def __init__(self, **kwargs) -> None:
        if "text" in kwargs:
            self.text = Text.from_dict(kwargs.get("text"))
        elif "image" in kwargs:
            self.image = Image.from_dict(kwargs.get("image"))
        elif "custom" in kwargs:
            self.custom = Custom.from_dict(kwargs.get("custom"))
        elif "video" in kwargs:
            self.video = Video.from_dict(kwargs.get("video"))
        elif "file" in kwargs:
            self.file = File.from_dict(kwargs.get("file"))
        elif "voice" in kwargs:
            self.voice = Voice.from_dict(kwargs.get("voice"))
        elif "share_link" in kwargs:
            self.share_link = ShareLink.from_dict(kwargs.get("share_link"))
        else:
            for k, v in kwargs.items():
                setattr(self, k, v)
            # raise ValueError("err msg body value")



from typing import List

from wulaisdk.response import BaseModel
from wulaisdk.response.bot import Response


class QA(BaseModel):
    knowledge_id: int
    standard_question: str
    question: str

    def __init__(self, knowledge_id: int, standard_question: str, question: str) -> None:
        self.knowledge_id = knowledge_id
        self.standard_question = standard_question
        self.question = question


class QASuggestedResponse(BaseModel):
    qa: QA
    is_send: bool
    score: int
    response: List[Response]
    quick_reply: List[str]

    def __init__(self, qa: QA, is_send: bool, score: int, response: List[Response], quick_reply: List[str]) -> None:
        self.qa = QA.from_dict(qa)
        self.is_send = is_send
        self.score = score
        self.response = [Response.from_dict(r) for r in response]
        self.quick_reply = quick_reply


class QABotResponse(BaseModel):
    is_dispatch: bool
    msg_id: str
    qa_suggested_response: List[QASuggestedResponse]
    extra: str

    def __init__(self, is_dispatch: bool, msg_id: str, qa_suggested_response: List[QASuggestedResponse], extra: str) -> None:
        self.is_dispatch = is_dispatch
        self.msg_id = msg_id
        self.qa_suggested_response = [QASuggestedResponse.from_dict(qsr) for qsr in qa_suggested_response]
        self.extra = extra



"""
基础对话场景


实现效果：发送消息给机器人，得到相应回复。

对接前提：
1. 调用方提供接入机器人的渠道。
2. 已经在吾来机器人平台搭建并训练了机器人。

对接步骤：
1. 调用 user/create 接口，传入 user_id，创建用户；
2. 调用 msg/bot-response 接口，传入用户消息和 user_id；
3. 机器人会返回1~10条可能的回复，按照置信度从高到低排序。
   调用方可根据自己业务需要，依据置信度参数 score 的高低或者 is_send 的值，选择一条或多条回复。
"""
import os

from wulaisdk.client import WulaiClient


pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")

client = WulaiClient(pubkey, secret, debug=False)

# 创建用户
resp_cu = client.create_user("test_user", nickname="测试用户")

# 获取回复
msg_body = {
    "text": {"content": "你好"}
}
resp_gbr = client.get_bot_response("test_user", msg_body)



"""
自定义接口地址，参数
"""
import os

from wulaisdk.client import WulaiClient
from wulaisdk.request import CommonRequest


pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")

client = WulaiClient(pubkey, secret, debug=False)

action = "/user/create"
params = {
    "user_id": "tet_user",
    "avatar_url": "",
    "nickname": "testUser"
}
opts = {
    "method": "POST",
    "timeout": 3,
    "retry": 0
}
request = CommonRequest(action, params, opts)
resp = client.process_common_request(request)



import requests
from requests.adapters import HTTPAdapter


class BaseRequest:

    _session = None

    def __init__(self, endpoint: str="https://openapi.wul.ai", method: str="POST",
                 pool_connections: int=10, pool_maxsize: int=10, max_retries: int=3):
        self.endpoint = endpoint
        self.method = method
        self.pool_connections = pool_connections
        self.pool_maxsize = pool_maxsize
        self.max_retries = max_retries

    def init_session(self):
        session = requests.Session()
        adapter = HTTPAdapter(
            pool_connections=self.pool_connections,
            pool_maxsize=self.pool_maxsize,
            max_retries=self.max_retries
        )
        session.mount(self.endpoint, adapter)
        self._session = session

    def post(self, url: str, data: dict, headers: dict, timeout: int=3, **kwargs):
        if self._session is None:
            self.init_session()
        resp = self._session.post(url, json=data, headers=headers, timeout=timeout, **kwargs)
        return resp

    def get(self, url: str, params: dict, headers: dict, timeout: int=3, **kwargs):
        if self._session is None:
            self.init_session()
        resp = self._session.get(url, params=params, headers=headers, timeout=timeout, **kwargs)
        return resp



from typing import List
from wulaisdk.response import BaseModel
from wulaisdk.response.msg_body import MsgBody


class SenderInfo(BaseModel):
    avatar_url: str
    nickname: str
    real_name: str

    def __init__(self, avatar_url: str, nickname: str, real_name: str) -> None:
        self.avatar_url = avatar_url
        self.nickname = nickname
        self.real_name = real_name


class UserInfo(BaseModel):
    avatar_url: str
    nickname: str

    def __init__(self, avatar_url: str, nickname: str) -> None:
        self.avatar_url = avatar_url
        self.nickname = nickname


class Msg(BaseModel):
    direction: str
    sender_info: SenderInfo
    msg_type: str
    extra: str
    msg_id: str
    msg_ts: str
    user_info: UserInfo
    msg_body: MsgBody

    def __init__(self, direction: str, sender_info: SenderInfo, msg_type: str, extra: str, msg_id: str, msg_ts: str, user_info: UserInfo, msg_body: MsgBody) -> None:
        self.direction = direction
        if sender_info:
            self.sender_info = SenderInfo.from_dict(sender_info)
        else:
            self.sender_info = SenderInfo("", "", "")
        self.msg_type = msg_type
        self.extra = extra
        self.msg_id = msg_id
        self.msg_ts = msg_ts
        if user_info:
            self.user_info = UserInfo.from_dict(user_info)
        else:
            self.user_info = UserInfo("", "")
        self.msg_body = MsgBody.from_dict(msg_body)


class HistoryMessage(BaseModel):
    msg: List[Msg]
    has_more: bool

    def __init__(self, msg: List[Msg], has_more: bool) -> None:
        self.msg = [Msg.from_dict(m) for m in msg]
        self.has_more = has_more



"""
相似问
"""
from typing import List

from wulaisdk.response import BaseModel


class SimilarQuestion(BaseModel):
    knowledge_id: str
    question: str
    id: str

    def __init__(self, knowledge_id: str, question: str, id: str) -> None:
        self.knowledge_id = knowledge_id
        self.question = question
        self.id = id


class SimilarQuestionCreate(BaseModel):
    similar_question: SimilarQuestion

    def __init__(self, similar_question: SimilarQuestion) -> None:
        self.similar_question = SimilarQuestion.from_dict(similar_question)


class SimilarQuestionUpdate(BaseModel):
    similar_question: SimilarQuestion

    def __init__(self, similar_question: SimilarQuestion) -> None:
        self.similar_question = SimilarQuestion.from_dict(similar_question)


class SimilarQuestions(BaseModel):
    similar_questions: List[SimilarQuestion]
    page_count: int

    def __init__(self, similar_questions: List[SimilarQuestion], page_count: int) -> None:
        self.similar_questions = [SimilarQuestion.from_dict(sq) for sq in similar_questions]
        self.page_count = page_count



"""
任务类api测试
"""
import os
import sys
import time
import pytest

from wulaisdk.client import WulaiClient


pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")
log_dir_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# 区分3.6和3.7的测试内容
if sys.version_info.minor == 7:
    tag = "v3.7"
else:
    tag = "v3.6"


#############################################################
#                           任务类
#############################################################
# 场景
@pytest.mark.parametrize('scene_create, scene_update', [
    # 标准场景创建
    ({
        "intent_switch_mode": "INTENT_SWITCH_MODE_STAY",
        "name": "测试场景-优先填槽",
        "smart_slot_filling_threshold": 0.7,
        "description": "测试场景-优先填槽"
     }, {
        "intent_switch_mode": "INTENT_SWITCH_MODE_STAY",
        "name": "测试场景-优先填槽-update",
        "smart_slot_filling_threshold": 0.69,
        "description": "测试场景-优先填槽-update",
        "id": ""
     }),
    # 不传智能填槽阈值
    ({
        "intent_switch_mode": "INTENT_SWITCH_MODE_SWITCH",
        "name": "测试场景-优先跳转",
        "description": "测试场景-优先跳转"
     }, {
        "intent_switch_mode": "INTENT_SWITCH_MODE_SWITCH",
        "name": "测试场景-优先跳转-update",
        "smart_slot_filling_threshold": 0.69,
        "description": "测试场景-优先跳转-update",
        "id": ""
     }),
    # 修改时不修改智能填槽阈值
    ({
        "intent_switch_mode": "INTENT_SWITCH_MODE_STAY",
        "name": "测试场景-优先填槽2",
        "smart_slot_filling_threshold": 0.69,
        "description": "测试场景-优先填槽"
     }, {
        "intent_switch_mode": "INTENT_SWITCH_MODE_STAY",
        "name": "测试场景-优先填槽2-update",
        "id": ""
     }),
])
def test_scenes(scene_create, scene_update):
    client = WulaiClient(pubkey, secret, debug=True)

    # 创建场景
    resp_create = client.create_scene(scene_create)
    scene_id = resp_create.scene.id

    # 更新场景
    scene_update["id"] = scene_id
    resp_update = client.update_scene(scene_update)

    assert scene_id == resp_update.scene.id

    # 查询场景列表
    resp_list = client.scenes()
    scene_ids = [scene.id for scene in resp_list.scenes]

    assert scene_id in scene_ids

    # 删除场景
    client.delete_scene(scene_id)

    # 查询场景列表
    resp_list = client.scenes()
    scene_ids = [scene.id for scene in resp_list.scenes]
    assert scene_id not in scene_ids


# 测试场景id： 12379
test_scene_id = 12379
# 测试意图id： 52034
test_intent_id = 52034
# 测试词槽id:
test_slot_id = 79396


# 意图
@pytest.mark.parametrize('intent_create, intent_update', [
    # 标准意图创建
    ({
        "scene_id": test_scene_id,
        "name": "测试意图",
        "lifespan_mins": 5
     }, {
        "id": "",
        "name": "测试意图-update",
        "lifespan_mins": 4
     }),
    # 标准意图创建-不传意图保留时间
    ({
        "scene_id": test_scene_id,
        "name": "测试意图2",
     }, {
        "id": "",
        "name": "测试意图2-update",
        "lifespan_mins": 4
     })
])
def test_intents(intent_create, intent_update):
    client = WulaiClient(pubkey, secret, debug=True)

    # 创建意图
    resp_create = client.create_intent(intent_create)
    intent_id = resp_create.intent.id

    # 更新意图
    intent_update["id"] = intent_id
    resp_update = client.update_intent(intent_update)

    assert intent_id == resp_update.intent.id

    # 查询意图列表
    resp_list = client.intents(intent_create["scene_id"])
    intent_ids = [intent.id for intent in resp_list.intents]

    assert intent_id in intent_ids

    # 删除意图
    client.delete_intent(intent_id)

    # 查询意图列表
    resp_list = client.intents(test_scene_id)
    intent_ids = [intent.id for intent in resp_list.intents]

    assert intent_id not in intent_ids


# 触发器
@pytest.mark.parametrize('intent_trigger_create, intent_trigger_update', [
    # 关键词匹配触发器创建
    ({
        "text": "关键词匹配触发文本",
        "intent_id": test_intent_id,
        "type": "TRIGGER_TYPE_EXACT_MATCH_KEYWORD"
     }, {
        "id": "",
        "text": "关键词触发文本更新"
     }),
    # 关键词包含触发器创建
    ({
        "text": "关键词包含触发文本",
        "intent_id": test_intent_id,
        "type": "TRIGGER_TYPE_INCLUDE_KEYWORD"
     }, {
        "id": "",
        "text": "关键词包含触发文本更新"
     }),
    # 相似说法触发器创建
    ({
        "text": "相似说法触发文本",
        "intent_id": test_intent_id,
        "type": "TRIGGER_TYPE_SENTENCE"
     }, {
        "id": "",
        "text": "相似说法触发文本更新"
     }),
])
def test_intent_trigger(intent_trigger_create, intent_trigger_update):
    client = WulaiClient(pubkey, secret, debug=True)

    # 创建触发器
    resp_create = client.create_intent_trigger(intent_trigger_create)
    intent_trigger_id = resp_create.intent_trigger.id

    # 更新触发器
    intent_trigger_update["id"] = intent_trigger_id
    resp_update = client.update_intent_trigger(intent_trigger_update)

    assert intent_trigger_id == resp_update.intent_trigger.id

    # 查询触发器列表
    resp_list = client.intent_triggers(intent_trigger_create["intent_id"], 1, 50)
    intent_trigger_ids = [intent_trigger.id for intent_trigger in resp_list.intent_triggers]

    assert intent_trigger_id in intent_trigger_ids

    # 删除触发器
    client.delete_intent_trigger(intent_trigger_id)

    # 查询触发器列表
    resp_list = client.intent_triggers(intent_trigger_create["intent_id"], 1, 50)
    intent_trigger_ids = [intent_trigger.id for intent_trigger in resp_list.intent_triggers]

    assert intent_trigger_id not in intent_trigger_ids


# 词槽
# 创建词槽背后应该是异步的，收到response时不代表词槽已经创建。
@pytest.mark.parametrize('slot_create, slot_update, data_source', [
    # 词槽
    ({
        "scene_id": test_scene_id,
        "name": f"词槽1-{tag}",
        "query_slot_filling": False
     }, {
        "id": "",
        "name": f"词槽1-{tag}",
        "query_slot_filling": True
     }, {
        "entity_id": 2,  # 星座
        "slot_id": ""
    }),
    # 词槽更新-不传name
    ({
        "scene_id": test_scene_id,
        "name": f"词槽2-3.{sys.version_info.minor}",
     }, {
        "id": "",
        "query_slot_filling": True
     }, {
        "entity_id": 2,  # 星座
        "slot_id": ""
    })
])
def test_slot(slot_create, slot_update, data_source):
    client = WulaiClient(pubkey, secret, debug=False)

    # 创建词槽
    resp_create = client.create_slot(slot_create)
    slot_id = resp_create.slot.id

    # 创建词槽数据来源
    data_source["slot_id"] = slot_id
    resp_data_source_create = client.create_slot_data_source(data_source)
    data_source_id = resp_data_source_create.data_source.id
    assert resp_data_source_create.data_source.entity_id == 2
    assert data_source_id

    # 查询词槽数据来源
    resp_data_source = client.slot_data_source(slot_id)
    slot_ids_ds = [one_data_source.slot_id for one_data_source in resp_data_source.data_sources]
    assert slot_id in slot_ids_ds

    # 删除词槽数据来源
    client.delete_slot_data_source(data_source_id)

    # 更新词槽
    slot_update["id"] = slot_id
    time.sleep(1)
    resp_update = client.update_slot(slot_update)

    assert slot_id == resp_update.slot.id

    # 查询词槽
    slot = client.get_slot(slot_id)
    assert slot.slot.name

    # 查询词槽列表
    resp_list = client.slots(slot_create["scene_id"], 1, 50)
    slot_ids = [slot.id for slot in resp_list.slots]

    assert slot_id in slot_ids

    # 删除词槽
    client.delete_slot(slot_id)

    # 查询词槽列表
    resp_list = client.slots(slot_create["scene_id"], 1, 50)
    slot_ids = [slot.id for slot in resp_list.slots]

    assert slot_id not in slot_ids


# 单元
# 消息发送单元
@pytest.mark.parametrize('inform_block_create, inform_block_update, response_create, response_update', [
    # 常规
    ({
        "intent_id": test_intent_id,
        "name": "消息发送单元1",
        "mode": "RESPONSE_ALL"
     }, {
        "id": "",
        "name": "消息发送单元1-update",
        "mode": "RESPONSE_ALL"
     }, {
        "block_id": "",
        "response": {
            "text": {
                "content": "这是消息发送单元"
            }
        }
    }, {
        "id": "",
        "response": {
            "image": {
                "resource_url": "https://www.baidu.com/img/bd_logo1.png"
            }
        }
    }),
    ({
        "intent_id": test_intent_id,
        "name": "消息发送单元2",
        "mode": "RESPONSE_LOOP"
     }, {
        "id": "",
        "name": "消息发送单元2-update",
        "mode": "RESPONSE_RANDOM"
     }, {
        "block_id": "",
        "response": {
            # "rich_text": {
            #     "resource_url": "https://www.baidu.com/img/bd_logo1.png"
            # }
            "custom": {
                "content": "测试custom自定义消息"
            }
        }
    }, {
        "id": "",
        "response": {
            "text": {
                "content": "https://www.baidu.com/img/bd_logo1.png"
            }
        }
    }),
])
def test_inform_block(inform_block_create, inform_block_update, response_create, response_update):
    client = WulaiClient(pubkey, secret, debug=True)

    # 创建消息发送单元
    resp_create = client.create_inform_block(inform_block_create)
    inform_block_id = resp_create.block.id

    # 更新消息发送单元
    inform_block_update["id"] = inform_block_id
    resp_update = client.update_inform_block(inform_block_update)

    assert inform_block_id == resp_update.block.id

    # 查询消息发送单元
    resp = client.inform_block(inform_block_id)

    assert resp.block.name

    # 创建单元内回复
    response_create["block_id"] = inform_block_id
    resp_resp_create = client.create_block_response(response_create)
    assert resp_resp_create.response.response.to_dict()
    key = list(resp_resp_create.response.response.__dict__.keys())[0]
    assert getattr(resp_resp_create.response.response, key).to_dict()

    resp_id = resp_resp_create.response.id

    # 更新单元内回复
    response_update["id"] = resp_id
    resp_resp_update = client.update_block_response(response_update)
    assert resp_resp_update.response.response.to_dict()
    key = list(resp_resp_update.response.response.__dict__.keys())[0]
    assert getattr(resp_resp_update.response.response, key).to_dict()

    # 删除单元内回复
    client.delete_block_response(resp_id)

    # 删除单元
    client.delete_block(inform_block_id)


# 询问填槽单元
@pytest.mark.parametrize('request_block_create, request_block_update, response_create', [
    # 常规
    ({
        "intent_id": test_intent_id,
        "name": "询问填槽单元1",
        "mode": "RESPONSE_ALL",
        "default_slot_value": "_default",
        "slot_filling_when_asked": False,
        "request_count": 2,
        "slot_id": test_slot_id
     }, {
        "id": "",
        "name": "询问填槽单元1-update",
        "slot_filling_when_asked": True,
     }, {
        "block_id": "",
        "response": {
            "text": {
                "content": "这是询问填槽单元"
            }
        }
    }),
])
def test_request_block(request_block_create, request_block_update, response_create):
    client = WulaiClient(pubkey, secret, debug=True)

    # 创建询问填槽单元
    resp_create = client.create_request_block(request_block_create)
    block_id = resp_create.block.id

    # 更新询问填槽单元
    request_block_update["id"] = block_id
    resp_update = client.update_request_block(request_block_update)

    assert block_id == resp_update.block.id

    # 查询询问填槽单元
    resp = client.request_block(block_id)

    assert resp.block.name

    # 创建单元内回复
    response_create["block_id"] = block_id
    resp_resp_create = client.create_block_response(response_create)
    assert resp_resp_create.response.response.to_dict()
    key = list(resp_resp_create.response.response.__dict__.keys())[0]
    assert getattr(resp_resp_create.response.response, key).to_dict()

    resp_id = resp_resp_create.response.id

    # 删除单元内回复
    client.delete_block_response(resp_id)

    # 删除单元
    client.delete_block(block_id)


# 意图终点单元
@pytest.mark.smoke
@pytest.mark.parametrize('end_block_create, end_block_update_1, end_block_update_2', [
    # 常规
    ({
        "intent_id": "",
        "name": "终点单元1",
        "slot_memorizing": False,
        "action": {"last": {}}
     }, {
        "id": "",
        "name": "终点单元1-update",
        "slot_memorizing": True,
        "action": {"end": {}}
     }, {
        "id": "",
        "name": "终点单元1-update",
        "slot_memorizing": True,
        "action": {"specified": {"id": test_intent_id}}
    }),
])
def test_end_block(end_block_create, end_block_update_1, end_block_update_2):
    client = WulaiClient(pubkey, secret, debug=True)

    # 创建意图
    intent = {
        "scene_id": test_scene_id,
        "name": "测试意图3",
        "lifespan_mins": 5
    }
    resp_create_intent = client.create_intent(intent)
    intent_id = resp_create_intent.intent.id

    end_block_create["intent_id"] = intent_id

    # 创建意图终点单元
    resp_create = client.create_end_block(end_block_create)
    block_id = resp_create.block.id

    # 更新意图终点单元
    end_block_update_1["id"] = block_id
    end_block_update_2["id"] = block_id
    resp_update_1 = client.update_end_block(end_block_update_1)
    resp_update_2 = client.update_end_block(end_block_update_2)

    assert block_id == resp_update_1.block.id
    assert block_id == resp_update_2.block.id

    # 查询询问填槽单元
    resp = client.end_block(block_id)

    assert resp.block.name

    # 删除单元
    client.delete_block(block_id)

    # 删除意图
    client.delete_intent(intent_id)


# 待审核消息
def test_intent_trigger_learning():
    client = WulaiClient(pubkey, secret, debug=True)

    # 查询任务待审核消息列表
    resp_list = client.intent_trigger_learning(1, 50)
    assert resp_list.to_dict()
    query_item = resp_list.query_items[0]
    assert query_item.recommend_intent.intent_name
    intent_trigger_learning_id = query_item.id

    # 删除任务待审核消息
    client.delete_intent_trigger_learning(intent_trigger_learning_id)


# 完整意图测试: 单元列表、单元关系、更新意图状态
def test_whole():
    client = WulaiClient(pubkey, secret, debug=True)

    # 创建意图
    intent = {
        "scene_id": test_scene_id,
        "name": "全流程测试意图",
        "lifespan_mins": 5
    }
    resp_create_intent = client.create_intent(intent)
    intent_id = resp_create_intent.intent.id

    # 创建消息发送单元
    inform_block_1 = {
        "intent_id": intent_id,
        "name": "消息发送1",
        "mode": "RESPONSE_ALL"
    }
    inform_block_2 = {
        "intent_id": intent_id,
        "name": "消息发送2",
        "mode": "RESPONSE_ALL"
    }
    resp_inform_block_id1 = client.create_inform_block(inform_block_1)
    resp_inform_block_id2 = client.create_inform_block(inform_block_2)
    inform_block_1_id = resp_inform_block_id1.block.id
    inform_block_2_id = resp_inform_block_id2.block.id

    # 创建单元内回复
    response_1_1 = {
        "block_id": inform_block_1_id,
        "response": {
            "text": {
                "content": "回复1_1"
            }
        }
    }
    response_1_2 = {
        "block_id": inform_block_1_id,
        "response": {
            "text": {
                "content": "回复1_2"
            }
        }
    }
    response_2_1 = {
        "block_id": inform_block_2_id,
        "response": {
            "text": {
                "content": "回复2_1"
            }
        }
    }
    client.create_block_response(response_1_1)
    client.create_block_response(response_1_2)
    client.create_block_response(response_2_1)

    # 创建询问单元
    request_block = {
        "intent_id": intent_id,
        "name": "询问单元1",
        "mode": "RESPONSE_ALL",
        "default_slot_value": "_default",
        "slot_filling_when_asked": False,
        "request_count": 2,
        "slot_id": test_slot_id
     }
    resp_request_block = client.create_request_block(request_block)
    request_block_id = resp_request_block.block.id
    # 创建单元内回复
    response_1_1 = {
        "block_id": request_block_id,
        "response": {
            "text": {
                "content": "询问1:"
            }
        }
    }
    client.create_block_response(response_1_1)

    # 创建意图终点单元
    end_block = {
        "intent_id": intent_id,
        "name": "终点单元1",
        "slot_memorizing": False,
        "action": {"end": {}}
     }
    resp_end_block = client.create_end_block(end_block)
    end_block_id = resp_end_block.block.id

    # 创建单元关系
    # 1.消息单元1->询问单元
    relation_1 = {
        "connection": {
            "from_block_id": inform_block_1_id,
            "to_block_id": request_block_id
        },
        "intent_id": intent_id
    }
    resp_relation_1 = client.create_block_relation(relation_1)
    assert resp_relation_1.relation.id

    # 2. 询问单元->消息单元2
    relation_2 = {
        "connection": {
            "from_block_id": request_block_id,
            "to_block_id": inform_block_2_id,
            "condition": {"less_than_or_equal_to": {"value": 50}}
        },
        "intent_id": intent_id
    }
    resp_relation_2 = client.create_block_relation(relation_2)
    assert resp_relation_2.relation.intent_id

    # 3. 询问单元->意图终点单元
    relation_3 = {
        "connection": {
            "from_block_id": request_block_id,
            "to_block_id": end_block_id,
            "condition": {"greater_than": {"value": 50}}
        },
        "intent_id": intent_id
    }
    resp_relation_3 = client.create_block_relation(relation_3)
    assert resp_relation_3.relation.connection.condition.greater_than.value

    # 4. 询问单元->消息单元2
    relation_4 = {
        "connection": {
            "from_block_id": request_block_id,
            "to_block_id": inform_block_2_id,
            "condition": {"default": {}}
        },
        "intent_id": intent_id
    }
    resp_relation_4 = client.create_block_relation(relation_4)
    relation_4_id = resp_relation_4.relation.id
    # 5. 消息单元2->意图终点单元
    relation_5 = {
        "connection": {
            "from_block_id": inform_block_2_id,
            "to_block_id": end_block_id
        },
        "intent_id": intent_id
    }
    resp_relation_5 = client.create_block_relation(relation_5)
    assert resp_relation_5.relation.connection.from_block_id

    # 更新意图状态
    status = True
    first_block_id = inform_block_1_id

    client.update_intent_status(status, first_block_id, intent_id)

    # 删除单元关系
    client.delete_block_relation(relation_4_id)

    # 查询单元列表
    resp_list = client.blocks(intent_id, 1, 50)
    assert resp_list.blocks
    block_ids = [block.id for block in resp_list.blocks]
    assert inform_block_1_id in block_ids
    assert inform_block_2_id in block_ids
    assert request_block_id in block_ids
    assert end_block_id in block_ids

    # 删除意图
    client.delete_intent(intent_id)



from typing import List

from wulaisdk.response import BaseModel
from wulaisdk.response.bot import Response


class Entity(BaseModel):
    idx_end: int
    name: str
    idx_start: int
    value: str
    seg_value: str
    type: str
    desc: str

    def __init__(self, idx_end: int, name: str, idx_start: int, value: str, seg_value: str, type: str, desc: str) -> None:
        self.idx_end = idx_end
        self.name = name
        self.idx_start = idx_start
        self.value = value
        self.seg_value = seg_value
        self.type = type
        self.desc = desc


class Task(BaseModel):
    block_type: str
    block_id: int
    task_id: int
    block_name: str
    entities: List[Entity]
    task_name: str
    robot_id: int

    def __init__(self, block_type: str, block_id: int, task_id: int, block_name: str, entities: List[Entity],
                 task_name: str, robot_id: int) -> None:
        self.block_type = block_type
        self.block_id = block_id
        self.task_id = task_id
        self.block_name = block_name
        self.entities = [Entity.from_dict(entity) for entity in entities]
        self.task_name = task_name
        self.robot_id = robot_id


class TaskSuggestedResponse(BaseModel):
    score: int
    is_send: bool
    task: Task
    response: List[Response]
    quick_reply: List[str]

    def __init__(self, score: int, is_send: bool, task: Task, response: List[Response], quick_reply: List[str]) -> None:
        self.score = score
        self.is_send = is_send
        self.task = Task.from_dict(task)
        self.response = [Response.from_dict(r) for r in response]
        self.quick_reply = quick_reply


class TaskBotResponse(BaseModel):
    is_dispatch: bool
    msg_id: str
    task_suggested_response: List[TaskSuggestedResponse]
    extra: str

    def __init__(self, is_dispatch: bool, msg_id: str, task_suggested_response: List[TaskSuggestedResponse], extra: str) -> None:
        self.is_dispatch = is_dispatch
        self.msg_id = msg_id
        self.task_suggested_response = [TaskSuggestedResponse.from_dict(tsr) for tsr in task_suggested_response]
        self.extra = extra



from typing import List

from wulaisdk.response import BaseModel
from wulaisdk.response.msg_body import MsgBody


class Entity(BaseModel):
    # todo: type
    idx_end: int
    name: str
    idx_start: int
    value: str
    seg_value: str
    type: str
    desc: str

    def __init__(self, idx_end: int, name: str, idx_start: int, value: str, seg_value: str,
                 type: str, desc: str) -> None:
        self.idx_end = idx_end
        self.name = name
        self.idx_start = idx_start
        self.value = value
        self.seg_value = seg_value
        self.type = type
        self.desc = desc


class QA(BaseModel):
    knowledge_id: int
    standard_question: str
    question: str

    def __init__(self, knowledge_id: int, standard_question: str, question: str) -> None:
        self.knowledge_id = knowledge_id
        self.standard_question = standard_question
        self.question = question


class ChitChat(BaseModel):
    corpus: str

    def __init__(self, corpus: str) -> None:
        self.corpus = corpus


class Task(BaseModel):
    block_type: str
    block_id: int
    task_id: int
    block_name: str
    entities: List[Entity]
    task_name: str
    robot_id: int

    def __init__(self, block_type: str, block_id: int, task_id: int, block_name: str, entities: List[Entity],
                 task_name: str, robot_id: int) -> None:
        self.block_type = block_type
        self.block_id = block_id
        self.task_id = task_id
        self.block_name = block_name
        self.entities = [Entity.from_dict(entity) for entity in entities]
        self.task_name = task_name
        self.robot_id = robot_id


class KeyWord(BaseModel):
    keyword_id: int
    keyword: str

    def __init__(self, keyword_id: int, keyword: str) -> None:
        self.keyword_id = keyword_id
        self.keyword = keyword


class Bot(BaseModel):

    def __init__(self, **kwargs) -> None:
        if "qa" in kwargs:
            self.qa = QA.from_dict(kwargs.get("qa"))
        elif "chitchat" in kwargs:
            self.chit_chat = ChitChat.from_dict(kwargs.get("chitchat"))
        elif "task" in kwargs:
            self.task = Task.from_dict(kwargs.get("task"))
        elif "keyword" in kwargs:
            self.keyword = KeyWord.from_dict(kwargs.get("keyword"))
        else:
            for k, v in kwargs:
                setattr(self, k, v)
            # raise ValueError("err bot body value")


class SimilarResponse(BaseModel):
    url: str
    source: str
    detail: Bot

    def __init__(self, url: str, source: str, detail: Bot) -> None:
        self.url = url
        self.source = source
        self.detail = Bot.from_dict(detail)


class Response(BaseModel):
    msg_body: MsgBody
    similar_response: List[SimilarResponse]
    enable_evaluate: bool
    delay_ts: int
    extra: str
    answer_id: int

    def __init__(self, msg_body: MsgBody, similar_response: List[SimilarResponse], enable_evaluate: bool,
                 delay_ts: int, extra: str, answer_id: int) -> None:
        self.msg_body = MsgBody.from_dict(msg_body)
        self.similar_response = [SimilarResponse.from_dict(sr) for sr in similar_response]
        self.enable_evaluate = enable_evaluate
        self.delay_ts = delay_ts
        self.extra = extra
        self.answer_id = answer_id




"""
异步基础对话场景


通过异步方式实现用户与机器人的交互，比基础对话稍微复杂。实现效果：将用户发来的消息传到吾来，机器人处理后，回调消息投递接口返回机器人的回复。

对接前提：
1. 调用方提供消息收发的渠道。
2. 渠道具有 WebSocket 能力。
3. 已经在吾来平台-渠道设置-开放平台(新版)设置了消息投递接口。
4. 已经在吾来机器人平台搭建并训练了机器人。

对接步骤：
1. 调用 user/create，传入 user_id， 创建用户；
2. 如果需要个性化回复，在此处调用 user/user-attribute/create 接口，传入 user_id 和该用户的用户属性；
3. 调用 msg/receive 将用户的消息传给机器人；
4. 吾来机器人会根据吾来平台上的配置回调消息投递接口，返回置信度最高的回复，同时返回0~10条相近的回复，
   按照置信度从高到低排序。调用方可根据自己的业务流程，依据置信度参数 score 的高低或者 is_send 的值，选择一条或多条回复。
5. 吾来平台会回调消息投递接口，将最终回复发给用户。
"""
import os

from wulaisdk.client import WulaiClient


pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")

client = WulaiClient(pubkey, secret, debug=False)

# 创建用户
resp_cu = client.create_user("test_user", nickname="测试用户")

# 添加属性值 - 根据需要操作
# user_attribute_user_attribute_value = [
#     {"user_attribute": {"id": "101343"}, "user_attribute_value": {"name": "新用户"}},
#     {"user_attribute": {"id": "101344"}, "user_attribute_value": {"name": "小区a"}},
# ]
# resp_crra = client.create_user_user_attribute("test_user", user_attribute_user_attribute_value)

# 消息传递给机器人
msg_body = {
    "text": {"content": "你好"}
}
third_msg_id = "aaa"
user_id = "test_user"
resp_gbr = client.receive_message(user_id, msg_body, third_msg_id)

# 需要调用方配置消息投递接口，详见CALLBACK.md



import os
import sys
import pytest

from wulaisdk.client import WulaiClient
from wulaisdk.exceptions import ClientException


pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")
log_dir_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# 区分3.6和3.7的测试内容
if sys.version_info.minor == 7:
    tag = "v3.7"
else:
    tag = "v3.6"


#############################################################
#                          词库管理类
#############################################################
# 查看全部实体概要
@pytest.mark.parametrize('page, page_size', [
    (1, 50),
    (3, 50),
    (5, 50)
])
def test_dictionary_entities(page, page_size):
    client = WulaiClient(pubkey, secret, debug=True)

    resp = client.dictionary_entities(page, page_size)
    if not resp.entities:
        assert resp.to_dict()
        assert not resp.entities
    else:
        entity = resp.entities[-1]

        entity_id = entity.id

        resp_entity = client.dictionary_entity(entity_id)
    
        assert resp.to_dict()
        assert resp_entity.to_dict()


# 专有词汇
@pytest.mark.parametrize('term_item_create,term_item_update,page,page_size', [
    ({
        "term": {"name": f"yt_专有名词_1_{tag}"},
        "synonyms": [f"yt_专有名词_1", f"yt_名词_1", f"专有名词_1"]
    }, {
        "term": {"id": "", "name": f"yt_专有名词_1_update"},
        "synonyms": [f"yt_专有名词_1"]
    }, 1, 50),
])
def test_dictionary_term(term_item_create, term_item_update, page, page_size):
    client = WulaiClient(pubkey, secret, debug=True)

    # 创建专有词汇
    resp_create = client.create_dictionary_term(term_item_create)
    term_id = resp_create.term_item.term.id
    assert term_id

    # 更新专有词汇
    term_item_update["term"]["id"] = term_id
    resp_update = client.update_dictionary_term(term_item_update)
    term_id_2 = resp_update.term_item.term.id
    assert term_id == term_id_2

    # 专有词汇列表
    resp_list = client.dictionary_terms(page, page_size)
    term_ids = [term_item.term.id for term_item in resp_list.term_item]

    assert term_id in term_ids

    # 删除专有词汇
    client.delete_dictionary_term(term_id)


# 枚举实体
@pytest.mark.parametrize('enum_entity, entity_value, entity_value_part, entity_value_total', [
    ({"name": f"枚举实体测试1_{tag}"},
     {"synonyms": ["标准值1", "标准1", "准值1"], "standard_value": "标准值1"},
     {"synonyms": ["准值1"], "standard_value": "标准值1"},
     {"standard_value": "标准值1"},
     ),
])
def test_dictionary_entities_enum(enum_entity, entity_value, entity_value_part, entity_value_total):
    client = WulaiClient(pubkey, secret, debug=True)

    # 创建枚举实体
    resp_entity_enum = client.create_dictionary_entity_enumeration(enum_entity)
    entity_enum_id = resp_entity_enum.enum_entity.id

    # 创建枚举实体值
    resp_create = client.create_dictionary_entity_enumeration_value(entity_enum_id, entity_value)

    assert resp_create.enum_entity.id

    # 删除枚举实体部分值
    client.delete_dictionary_entity_enumeration_value(entity_enum_id, entity_value_part)

    # 删除枚举实体标准值
    client.delete_dictionary_entity_enumeration_value(entity_enum_id, entity_value_total)

    resp_entity = client.dictionary_entity(entity_enum_id)

    assert resp_entity.entity.name == enum_entity["name"]

    # 删除枚举实体
    client.delete_dictionary_entity(entity_enum_id)

    with pytest.raises(ClientException) as excinfo:
        client.dictionary_entity(entity_enum_id)
        assert excinfo.error_code == "SDK_INVALID_PARAMS"


# 意图实体
@pytest.mark.parametrize('intent_entity, entity_value_update, entity_value_delete', [
    ({"standard_value": "发烧", "name": f"意图实体测试1_{tag}"},
     ["体温有点高", "额头发热", "发烧"],
     ["额头发热"],
     ),
])
def test_dictionary_entities_intent(intent_entity, entity_value_update, entity_value_delete):
    client = WulaiClient(pubkey, secret, debug=True)

    # 创建意图实体
    resp_entity_intent = client.create_dictionary_entity_intent(intent_entity)
    entity_intent_id = resp_entity_intent.intent_entity.id

    # 创建意图实体值相似说法
    resp_create = client.create_dictionary_entity_intent_value(entity_intent_id, entity_value_update)

    assert entity_intent_id == resp_create.intent_entity.id

    # 删除意图实体值相似说法
    client.delete_dictionary_entity_intent_value(entity_intent_id, entity_value_delete)

    # 删除实体
    client.delete_dictionary_entity(entity_intent_id)

    with pytest.raises(ClientException) as excinfo:
        client.dictionary_entity(entity_intent_id)
        assert excinfo.error_code == "SDK_INVALID_PARAMS"




"""
知识点
"""
from typing import List

from wulaisdk.response import BaseModel


class Knowledge(BaseModel):
    status: bool
    update_time: str
    maintained_by_user_attribute_group: bool
    standard_question: str
    create_time: str
    respond_all: bool
    id: str

    def __init__(self, status: bool, update_time: str, maintained_by_user_attribute_group: bool,
                 standard_question: str, create_time: str, respond_all: bool, id: str) -> None:
        self.status = status
        self.update_time = update_time
        self.maintained_by_user_attribute_group = maintained_by_user_attribute_group
        self.standard_question = standard_question
        self.create_time = create_time
        self.respond_all = respond_all
        self.id = id


class KnowledgeTagKnowledge(BaseModel):
    knowledge: Knowledge
    knowledge_tag_id: str

    def __init__(self, knowledge: Knowledge, knowledge_tag_id: str) -> None:
        self.knowledge = Knowledge.from_dict(knowledge)
        self.knowledge_tag_id = knowledge_tag_id


# 创建知识点
class KnowledgeCreate(BaseModel):
    knowledge_tag_knowledge: KnowledgeTagKnowledge

    def __init__(self, knowledge_tag_knowledge: KnowledgeTagKnowledge) -> None:
        self.knowledge_tag_knowledge = KnowledgeTagKnowledge.from_dict(knowledge_tag_knowledge)


# 更新知识点
class KnowledgeUpdate(BaseModel):
    knowledge: Knowledge

    def __init__(self, knowledge: Knowledge) -> None:
        self.knowledge = Knowledge.from_dict(knowledge)


class KnowledgeTag(BaseModel):
    parent_knowledge_tag_id: str
    id: str
    name: str

    def __init__(self, parent_knowledge_tag_id: str, id: str, name: str) -> None:
        self.parent_knowledge_tag_id = parent_knowledge_tag_id
        self.id = id
        self.name = name


class SimilarQuestion(BaseModel):
    knowledge_id: str
    question: str
    id: str

    def __init__(self, knowledge_id: str, question: str, id: str) -> None:
        self.knowledge_id = knowledge_id
        self.question = question
        self.id = id


class KnowledgeItem(BaseModel):
    knowledge_tag: KnowledgeTag
    similar_questions: List[SimilarQuestion]
    knowledge: Knowledge

    def __init__(self, knowledge_tag: KnowledgeTag, similar_questions: List[SimilarQuestion], knowledge: Knowledge) -> None:
        self.knowledge_tag = KnowledgeTag.from_dict(knowledge_tag)
        self.similar_questions = [SimilarQuestion.from_dict(sq) for sq in similar_questions]
        self.knowledge = Knowledge.from_dict(knowledge)


# 查询知识点列表
class KnowledgeItems(BaseModel):
    page_count: int
    knowledge_items: List[KnowledgeItem]

    def __init__(self, page_count: int, knowledge_items: List[KnowledgeItem]) -> None:
        self.page_count = page_count
        self.knowledge_items = [KnowledgeItem.from_dict(ki) for ki in knowledge_items]


# 查询知识点分类列表
class KnowledgeTags(BaseModel):
    knowledge_tags: List[KnowledgeTag]
    page_count: int

    def __init__(self, knowledge_tags: List[KnowledgeTag], page_count: int) -> None:
        self.knowledge_tags = [KnowledgeTag.from_dict(kt) for kt in knowledge_tags]
        self.page_count = page_count



"""
个性化对话场景


在基础对话的基础上，对不同用户回复不同的内容。实现效果：发送消息给机器人，机器人根据用户的属性判断回复内容并作出回复。

对接前提：
1. 调用方提供接入机器人的渠道。
2. 已经在吾来机器人平台搭建并训练了机器人。
3. 已经在吾来机器人平台定义了用户属性。

对接步骤：
1. 调用 user/create 接口，传入 user_id，创建用户；
2. 调用 user/user-attribute/create 接口，传入 user_id 和该用户的用户属性；
3. 调用 msg/bot-response 接口，传入 user_id 和该用户发来的消息；
4. 机器人会返回1~10条可能的回复，按照置信度从高到低排序。
   调用方可根据自己业务需要，依据置信度参数 score 的高低或者 is_send 的值，选择一条或多条回复。
"""
import os

from wulaisdk.client import WulaiClient

pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")

client = WulaiClient(pubkey, secret, debug=False)

# 创建用户
resp_cu = client.create_user("test_user", nickname="测试用户")

# 添加属性值
user_attribute_user_attribute_value = [
    {"user_attribute": {"id": "101343"}, "user_attribute_value": {"name": "新用户"}},
    {"user_attribute": {"id": "101344"}, "user_attribute_value": {"name": "小区a"}},
]
resp_crra = client.create_user_user_attribute("test_user", user_attribute_user_attribute_value)

# 获取回复
msg_body = {
    "text": {"content": "你好"}
}
resp_gbr = client.get_bot_response("test_user", msg_body)



from wulaisdk.exceptions import ClientException, ERR_INFO


class CommonRequest:

    def __init__(self, action: str, params: dict, opts: dict):
        """
        :param action:
        :param params:
        :param opts:
        """
        self.path = ""
        self.headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
            "User-Agent": "wulai-openapi-sdk-python/v2-1.1.5 python/3.6 requests/2.22"
        }
        self.action = action
        self.params = params
        self.opts = opts
        self.check()

    def check(self):
        self.check_action()
        self.check_params()
        self.check_opts()

    def check_action(self):
        if not isinstance(self.action, str):
            raise ClientException("SDK_INVALID_ACTION", ERR_INFO["SDK_INVALID_ACTION"])

    def check_params(self):
        if not isinstance(self.params, dict):
            raise ClientException("SDK_INVALID_PARAMS", ERR_INFO["SDK_INVALID_PARAMS"])

    def check_opts(self):
        if not isinstance(self.opts, dict):
            raise ClientException("SDK_INVALID_PARAMS", ERR_INFO["SDK_INVALID_PARAMS"])

    def set_headers(self, headers: dict):
        self.headers = headers

    def add_headers(self, k, v):
        self.headers[k] = v

    def update_headers(self, headers):
        self.headers.update(headers)



"""
属性组
"""
from typing import List

from wulaisdk.response import BaseModel
from wulaisdk.response.msg_body import MsgBody


class UserAttribute(BaseModel):
    """
    用户属性
    """
    value_type: str
    use_in_user_attribute_group: bool
    type: str
    id: str
    name: str

    def __init__(self, value_type: str, use_in_user_attribute_group: bool, type: str, id: str, name: str) -> None:
        self.value_type = value_type
        self.use_in_user_attribute_group = use_in_user_attribute_group
        self.type = type
        self.id = id
        self.name = name


class UserAttributeValue(BaseModel):
    """
    用户属性值
    """
    id: str
    name: str

    def __init__(self, id: str, name: str) -> None:
        self.id = id
        self.name = name


class UserAttributeGroup(BaseModel):
    """
    用户属性组详情
    """
    id: str
    name: str

    def __init__(self, id: str, name: str) -> None:
        self.id = id
        self.name = name


class UserAttributeUserAttributeValue(BaseModel):
    """
    属性
    """
    user_attribute: UserAttribute
    user_attribute_value: UserAttributeValue

    def __init__(self, user_attribute: UserAttribute, user_attribute_value: UserAttributeValue) -> None:
        self.user_attribute = UserAttribute.from_dict(user_attribute)
        self.user_attribute_value = UserAttributeValue.from_dict(user_attribute_value)


class UserAttributeGroupItem(BaseModel):
    """
    用户属性组及属性
    """
    user_attribute_user_attribute_values: List[UserAttributeUserAttributeValue]
    user_attribute_group: UserAttributeGroup

    def __init__(self, user_attribute_user_attribute_values: List[UserAttributeUserAttributeValue],
                 user_attribute_group: UserAttribute) -> None:
        self.user_attribute_user_attribute_values = [
            UserAttributeUserAttributeValue.from_dict(uauav) for uauav in user_attribute_user_attribute_values
        ]
        self.user_attribute_group = UserAttributeGroup.from_dict(user_attribute_group)


class CreateUserAttributeGroup(BaseModel):
    """
    创建属性组
    """
    user_attribute_group_item: UserAttributeGroupItem

    def __init__(self, user_attribute_group_item: UserAttributeGroupItem) -> None:
        self.user_attribute_group_item = UserAttributeGroupItem.from_dict(user_attribute_group_item)


class UpdateUserAttributeGroup(BaseModel):
    """
    更新属性组
    """
    user_attribute_group_item: UserAttributeGroupItem

    def __init__(self, user_attribute_group_item: UserAttributeGroupItem) -> None:
        self.user_attribute_group_item = UserAttributeGroupItem.from_dict(user_attribute_group_item)


class UpdateUserAttributeGroupItems(BaseModel):
    """
    查询属性组及属性列表
    """
    page_count: int
    user_attribute_group_items: List[UserAttributeGroupItem]

    def __init__(self, page_count: int, user_attribute_group_items: List[UserAttributeGroupItem]) -> None:
        self.page_count = page_count
        self.user_attribute_group_items = [
            UserAttributeGroupItem.from_dict(uagi) for uagi in user_attribute_group_items
        ]


class Answer(BaseModel):
    """
    回复
    """
    knowledge_id: str
    msg_body: MsgBody
    id: str

    def __init__(self, knowledge_id: str, msg_body: MsgBody, id: str) -> None:
        self.knowledge_id = knowledge_id
        self.msg_body = MsgBody.from_dict(msg_body)
        self.id = id


class UserAttributeGroupAnswer(BaseModel):
    """
    属性组回复
    """
    answer: Answer
    user_attribute_group_id: str

    def __init__(self, answer: Answer, user_attribute_group_id: str) -> None:
        self.answer = Answer.from_dict(answer)
        self.user_attribute_group_id = user_attribute_group_id


class CreateUserAttributeGroupAnswer(BaseModel):
    """
    创建属性组回复
    """
    user_attribute_group_answer: UserAttributeGroupAnswer

    def __init__(self, user_attribute_group_answer: UserAttributeGroupAnswer) -> None:
        self.user_attribute_group_answer = UserAttributeGroupAnswer.from_dict(user_attribute_group_answer)


class UpdateUserAttributeGroupAnswer(BaseModel):
    """
    更新属性组回复
    """
    user_attribute_group_answer: UserAttributeGroupAnswer

    def __init__(self, user_attribute_group_answer: UserAttributeGroupAnswer) -> None:
        self.user_attribute_group_answer = UserAttributeGroupAnswer.from_dict(user_attribute_group_answer)


class UserAttributeGroupAnswers(BaseModel):
    """
    查询属性组回复列表
    """
    user_attribute_group_answers: List[UserAttributeGroupAnswer]
    page_count: int

    def __init__(self, user_attribute_group_answers: List[UserAttributeGroupAnswer], page_count: int) -> None:
        self.user_attribute_group_answers = [
            UserAttributeGroupAnswer.from_dict(uaga) for uaga in user_attribute_group_answers
        ]
        self.page_count = page_count


class UserAttributeTotal(BaseModel):
    """
    用户属性
    """
    name: str
    lifespan: int
    value_type: str
    use_in_user_attribute_group: bool
    type: str
    id: str

    def __init__(self, name: str, lifespan: int, value_type: str, use_in_user_attribute_group: bool, type: str,
                 id: str) -> None:
        self.name = name
        self.lifespan = lifespan
        self.value_type = value_type
        self.use_in_user_attribute_group = use_in_user_attribute_group
        self.type = type
        self.id = id


class UserAttributeTotalUserAttributeValue(BaseModel):
    """
    属性
    """
    user_attribute: UserAttributeTotal
    user_attribute_value: UserAttributeValue

    def __init__(self, user_attribute: UserAttributeTotal, user_attribute_value: UserAttributeValue) -> None:
        self.user_attribute = UserAttributeTotal.from_dict(user_attribute)
        self.user_attribute_value = UserAttributeValue.from_dict(user_attribute_value)


class UserAttributeTotalUserAttributeValues(BaseModel):
    """
    属性列表
    """
    user_attribute: UserAttributeTotal
    user_attribute_value: List[UserAttributeValue]

    def __init__(self, user_attribute: UserAttributeTotal, user_attribute_value: List[UserAttributeValue]) -> None:
        self.user_attribute = UserAttributeTotal.from_dict(user_attribute)
        self.user_attribute_value = [UserAttributeValue.from_dict(uav) for uav in user_attribute_value]


class UserUserAttribute(BaseModel):
    """
    用户属性值
    """
    user_attribute_user_attribute_values: List[UserAttributeTotalUserAttributeValue]

    def __init__(self, user_attribute_user_attribute_values: List[UserAttributeTotalUserAttributeValue]) -> None:
        self.user_attribute_user_attribute_values = [
            UserAttributeTotalUserAttributeValue.from_dict(uauav) for uauav in user_attribute_user_attribute_values
        ]


class UserAttributes(BaseModel):
    """
    用户属性列表
    """
    page_count: int
    user_attribute_user_attribute_values: List[UserAttributeTotalUserAttributeValues]

    def __init__(self, page_count: int,
                 user_attribute_user_attribute_values: List[UserAttributeTotalUserAttributeValues]) -> None:
        self.page_count = page_count
        self.user_attribute_user_attribute_values = [
            UserAttributeTotalUserAttributeValues.from_dict(uauav) for uauav in user_attribute_user_attribute_values
        ]



"""
异步基础对话场景


通过异步方式实现更复杂的定制化对话。
实现效果：调用方将用户发送的消息传给吾来，吾来通过消息路由将机器人的回复发给调用方，调用方对回复内容进行二次加工
（如：修改机器人直接回复的内容、将机器人回复内容作为客服的候选回复等），
加工完成后再将最终要发送的回复内容传回消息路由，吾来通过消息投递接口将最终回复发送给用户。

对接前提：
1. 调用方提供消息收发的渠道。
2. 渠道具有 WebSocket 能力。
3. 已经在吾来平台-渠道设置-开放平台(新版)设置了消息路由接口。
4. 已经在吾来平台-渠道设置-开放平台(新版)设置了消息投递接口。
5. 已经在吾来机器人平台搭建并训练了机器人。

对接步骤：
1. 调用 user/create 接口，传入 user_id，创建用户；
2. 如果需要个性化回复，在此处调用 user/user-attribute/create 接口，传入 user_id 和该用户的用户属性；
3. 调用 msg/receive 接口，将用户的 query 发给吾来；
4. 在消息路由中接受机器人返回的结果，并根据业务需要修改相应内容；
  4.1 如果需修改机器人回复的内容，修改 msg_body 的内容。
  4.2 如果不希望这条消息发给用户，修改 is_send 为 false。
  4.3 如果希望将机器人回复作为候选回复展示给客服，可提取 suggested_response/response 中的内容。（该方法需要调用方的客服工作台支持前端展示页面）
  4.4 “是否派发”参数 is_dispatch 表示机器人判断是否应该转人工（true 代表应该转人工）。
      不管 is_dispatch 是 true 或 false ，调用方都可以将一条消息转人工处理。如果决定要做“转人工”处理，需要完成以下步骤：
      4.4.1 在消息路由的消息体中，修改 `is_send` 的值为 `false` ，这样这条消息就不会被发送给用户。
      4.4.2 同时将这条消息推送到调用方人工服务平台，并根据调用方规则安排客服。
      4.4.3 在人工接入状态下，接收的每一条用户消息，都需要分别做”转人工“处理。
  4.5 将消息路由的消息体处理完成之后，作为 Response 返回给吾来平台。
5. 吾来平台会回调消息投递接口，将最终回复发给用户。
"""
import os

from wulaisdk.client import WulaiClient


pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")

client = WulaiClient(pubkey, secret, debug=False)

# 创建用户
resp_cu = client.create_user("test_user", nickname="测试用户")

# 添加属性值 - 根据需要操作
# user_attribute_user_attribute_value = [
#     {"user_attribute": {"id": "101343"}, "user_attribute_value": {"name": "新用户"}},
#     {"user_attribute": {"id": "101344"}, "user_attribute_value": {"name": "小区a"}},
# ]
# resp_crra = client.create_user_user_attribute("test_user", user_attribute_user_attribute_value)

# 消息传递给机器人
msg_body = {
    "text": {"content": "你好"}
}
third_msg_id = "aaa"
user_id = "test_user"
resp_gbr = client.receive_message(user_id, msg_body, third_msg_id)

# 需要调用方配置消息路由接口，详见CALLBACK.md
# 需要调用方配置消息投递接口，详见CALLBACK.md



class ClientException(Exception):
    _error_type = "Client"

    def __init__(self, code, msg):
        super().__init__()
        self.error_code = code
        self.error_msg = msg

    def __str__(self):
        return "{}: {}".format(self.error_code, self.error_msg)


class ServerException(Exception):
    _error_type = "Server"

    def __init__(self, code, msg, http_status_code):
        super().__init__()
        self.error_code = code
        self.error_msg = msg
        self.http_status_code = http_status_code

    def __str__(self):
        return "{} {}: {}".format(self.http_status_code, self.error_code, self.error_msg)


ERR_INFO = {
    "SDK_SERVER_UNREACHABLE": "无法连接服务器",
    "SDK_INVALID_REQUEST": "The request is not a valid CommonRequest.",
    "SDK_INVALID_CREDENTIAL": "The secrect or pubkey is incorrect. Please check it",
    "SDK_INVALID_PARAMS": "The param is incorrect. Please check it",
    "SDK_INVALID_ACTION": "The action is incorrect. Please check it",
    "SDK_NOT_SUPPORT": "Invalid action, please check it",
    "SDK_HTTP_ERROR": "http request error",
    "SDK_METHOD_NOT_ALLOW": "Method not allow, please check it.",
    "SDK_INVALID_API_VERSION": "Invalid api version, please check it."
}



import os
import time
import pytest
import logging
import copy
import sys

from wulaisdk.client import WulaiClient
from wulaisdk.request import CommonRequest
from wulaisdk.exceptions import ClientException
from wulaisdk.response.bot_response import BotResponse
from wulaisdk.response.msg_body import MsgBody

pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")
log_dir_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


def find_value_from_dict_array(li, search_key, value_key, reference):
    """
    在[{}, {}, ...]中找到合适的值
    :param li:
    :param search_key: 需要查询的key
    :param value_key: 目标值所在key
    :param reference: 对比值
    :return:
    """
    for di in li:
        if di[search_key] == reference:
            return di[value_key]
    return None


@pytest.mark.parametrize('debug,action,params,expected', [
    (False, '/user/create',
     {
         "avatar_url": "",
         "user_id": "shierlou",
         "nickname": "测试用户-shierlou"
     },
     {}),
    (True, '/user/create',
     {
         "avatar_url": "",
         "user_id": "shierlou",
         "nickname": "测试用户-shierlou"
     },
     {}),
])
def test_common_request(debug, action, params, expected):
    opts = {
        "method": "POST",
        "timeout": 3,
        "retry": 0,
    }
    client = WulaiClient(pubkey, secret, debug=debug)
    request = CommonRequest(action, params, opts)
    resp = client.process_common_request(request)
    assert resp == expected


# test invalid http method
@pytest.mark.parametrize('debug,action,params,opts', [
    (False, '/user/create',
     {
         "avatar_url": "",
         "user_id": "shierlou",
         "nickname": "测试用户-shierlou"
     },
     {
         "method": "PUT",
         "timeout": 3,
         "retry": 0,
     }),
    (True, '/user/create',
     {
         "avatar_url": "",
         "user_id": "shierlou",
         "nickname": "测试用户-shierlou"
     },
     {
         "method": "GET",
         "timeout": 3,
         "retry": 0,
     }),
])
def test_http_method(debug, action, params, opts):
    client = WulaiClient(pubkey, secret, debug=debug)
    with pytest.raises(ClientException) as excinfo:
        request = CommonRequest(action, params, opts)
        client.process_common_request(request)
        assert excinfo.error_msg == "SDK_METHOD_NOT_ALLOW"


# test api_version
@pytest.mark.parametrize('debug,user_id,avatar_url,nickname', [
    (True, "shierlou", "", "测试用户-shierlou"),
    (True, "shierlou", "", ""),
])
def test_api_version(debug, user_id, avatar_url, nickname):
    with pytest.raises(ClientException) as excinfo:
        client = WulaiClient(pubkey, secret, debug=debug, api_version="v3")
        client.create_user(user_id, avatar_url, nickname)
        assert excinfo.error_code == "SDK_INVALID_API_VERSION"


# test log FileHandler
@pytest.mark.parametrize('handler', [
    logging.FileHandler(log_dir_path + '/logs/wulaisdk.log')
])
def test_log_handler(handler):
    client = WulaiClient(pubkey, secret, debug=True)
    client.add_logger_handler(handler)
    client.create_user("shierlou")


# test invalid credential
@pytest.mark.parametrize('debug,user_id,avatar_url,nickname', [
    (True, "shierlou", "", "测试用户-shierlou"),
    (True, "shierlou", "", ""),
])
def test_credential(debug, user_id, avatar_url, nickname):
    client = WulaiClient("asEGUJtCyLkHvTmm8vE0bXLe5ebGs2PP00df19808441f2cfff", secret, debug=debug)
    with pytest.raises(ClientException) as excinfo:
        client.create_user(user_id, avatar_url, nickname)
        assert excinfo.error_code == "SDK_INVALID_CREDENTIAL"


#############################################################
#                          用户类
#############################################################
# createUser test
@pytest.mark.parametrize('debug,user_id,avatar_url,nickname,expected', [
    (True, "shierlou", "", "测试用户-shierlou", {}),
    (True, "shierlou", "", "", {}),
])
def test_create_user_normal_1(debug, user_id, avatar_url, nickname, expected):
    client = WulaiClient(pubkey, secret, debug=debug)
    resp = client.create_user(user_id, avatar_url, nickname)
    assert resp == expected


@pytest.mark.parametrize('debug,user_id,expected', [
    (True, "shierlou", {}),
    (True, "shierlou", {}),
])
def test_create_user_normal_2(debug, user_id, expected):
    client = WulaiClient(pubkey, secret, debug=debug)
    resp = client.create_user(user_id)
    assert resp == expected


@pytest.mark.parametrize('debug,user_id', [
    (True, ""),
    (False, 123),
])
def test_create_user_error(debug, user_id):
    client = WulaiClient(pubkey, secret, debug=debug)
    with pytest.raises(ClientException) as excinfo:
        client.create_user(user_id)
        assert excinfo.error_code == "SDK_INVALID_PARAMS"


# 获取用户属性列表
@pytest.mark.parametrize('filter, page, page_size', [
    (None, 1, 50),
    (None, 2, 30),
    ({"use_in_user_attribute_group": True}, 1, 30),
    ({"use_in_user_attribute_group": False}, 1, 30)
])
def test_user_attributes(filter, page, page_size):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.user_attributes(page, page_size, filter)
    assert resp


# 查询用户属性值
@pytest.mark.parametrize('user_id', [
    "shierlou"
])
def test_user_user_attribute(user_id):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.user_user_attribute(user_id)
    assert resp


# 给用户添加属性值
@pytest.mark.parametrize('user_attribute_user_attribute_value, user_id', [
    ([{"user_attribute": {"id": "101627"}, "user_attribute_value": {"name": "微博"}}], "shierlou"),
    ([{"user_attribute": {"id": "101629"}, "user_attribute_value": {"name": "老用户"}}], "shierlou"),
    ([
        {"user_attribute": {"id": "101627"}, "user_attribute_value": {"name": "微信"}},
        {"user_attribute": {"id": "101629"}, "user_attribute_value": {"name": "新用户"}}
    ], "shierlou"),
])
def test_create_user_attribute(user_attribute_user_attribute_value, user_id):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.create_user_attribute(user_attribute_user_attribute_value, user_id)
    assert 1


#############################################################
#                          对话类
#############################################################
# getBotResponse test
@pytest.mark.parametrize('user_id,msg_body,extra', [
    ("shierlou", {"text": {"content": "你好"}}, "this is extra string")
])
def test_get_bot_response_normal(user_id, msg_body, extra):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_bot_response(user_id, msg_body, extra)
    assert resp.suggested_response[0].response[0].msg_body.text.content == '你好，有什么可以为你服务的吗？(*╹▽╹*)'


# getBotResponse for image test
@pytest.mark.parametrize('user_id,msg_body', [
    ("shierlou", {"text": {"content": "图片如何回复"}})
])
def test_get_bot_response_image(user_id, msg_body):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_bot_response(user_id, msg_body)
    assert isinstance(resp.suggested_response[0].response[0].msg_body.image.resource_url, str)


# createUserUserAttribute test
@pytest.mark.parametrize('user_id,user_attribute_user_attribute_value,expected', [
    ("shierlou", [
        {"user_attribute": {"id": "101343"}, "user_attribute_value": {"name": "新用户"}},
        {"user_attribute": {"id": "101344"}, "user_attribute_value": {"name": "小区a"}},
    ], {}),
])
def test_create_user_user_attribute_normal(user_id, user_attribute_user_attribute_value, expected):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.create_user_user_attribute(user_id, user_attribute_user_attribute_value)
    assert resp == expected


# getKeyWordBotResponse test
@pytest.mark.parametrize('user_id,msg_body,expected', [
    ("shierlou", {"text": {"content": "关键词测试"}}, "哈哈哈，这是关键词😃")
])
def test_get_keyword_bot_response_normal(user_id, msg_body, expected):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_keyword_bot_response(user_id, msg_body)
    assert resp.to_dict()["keyword_suggested_response"][0]["response"][0]["msg_body"]["text"]["content"] == expected
    assert resp.keyword_suggested_response[0].response[0].msg_body.text.content == expected


# getQABotResponse test
@pytest.mark.parametrize('user_id,msg_body,extra,expected', [
    ("shierlou", {"text": {"content": "吃了吗"}}, "this is extra string", "服务你第一，吃饭第二，请问有什么可以帮助你的吗？")
])
def test_get_qa_bot_response_normal(user_id, msg_body, extra, expected):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_qa_bot_response(user_id, msg_body, extra)
    assert resp.to_dict()["qa_suggested_response"][0]["response"][0]["msg_body"]["text"]["content"] == expected
    assert resp.qa_suggested_response[0].response[0].msg_body.text.content == expected


# getQABotResponse for image test
@pytest.mark.parametrize('user_id,msg_body,extra', [
    ("shierlou", {"text": {"content": "图片如何回复"}}, "this is extra string")
])
def test_get_qa_bot_response_image(user_id, msg_body, extra):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_qa_bot_response(user_id, msg_body, extra)
    assert isinstance(resp.qa_suggested_response[0].response[0].msg_body.image.resource_url, str)


# getTaskBotResponse test
@pytest.mark.parametrize('user_id,msg_body,extra', [
    ("shierlou", {"text": {"content": "我想查尺码"}}, "this is extra string")
])
def test_get_task_bot_response_normal(user_id, msg_body, extra):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_task_bot_response(user_id, msg_body, extra)
    assert "text" in resp.to_dict()["task_suggested_response"][0]["response"][0]["msg_body"].keys()
    assert isinstance(resp.task_suggested_response[0].response[0].msg_body.text.content, str)


# syncMessage test
@pytest.mark.parametrize('user_id,msg_body,msg_ts', [
    ("shierlou", {"text": {"content": "测试message_sync"}}, str(int(time.time() * 1000)))
])
def test_sync_message(user_id, msg_body, msg_ts):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.sync_message(user_id, msg_body, msg_ts)
    assert "msg_id" in resp.to_dict()
    assert isinstance(resp.msg_id, str)


# receiveMessage test
@pytest.mark.parametrize('user_id,msg_body', [
    ("shierlou", {"text": {"content": "测试receive_message"}})
])
def test_receive_message(user_id, msg_body):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.receive_message(user_id, msg_body)
    assert "msg_id" in resp.to_dict()
    assert isinstance(resp.msg_id, str)


# getMessageHistory test
@pytest.mark.parametrize('user_id,num', [
    ("shierlou", 50)
])
def test_message_history(user_id, num):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_message_history(user_id, num)
    assert resp.to_dict()
    assert isinstance(resp.msg[0].user_info.nickname, str)
    assert isinstance(resp.msg[0].msg_body.to_dict(), dict)
    msg_body = resp.msg[0].msg_body
    if hasattr(msg_body, "text"):
        assert isinstance(msg_body.text.content, str)
    elif hasattr(msg_body, "image"):
        assert isinstance(msg_body.image.resource_url, str)


# bot-response wrapper test
@pytest.mark.parametrize('body', [
    {"msg_id": "123", "test": "test", "is_dispatch": False, "suggested_response": [], "extra": ""}
])
def test_err_response(body):
    assert BotResponse.from_dict(body)


# msg_body wrapper test
@pytest.mark.parametrize('body', [
    {"text": {"content": "hhhh"}},
    {"image": {"resource_url": "hhhh"}},
    {"test": {"test": "test"}},
])
def test_err_response(body):
    assert MsgBody.from_dict(body)


#############################################################
#                         知识点类
#############################################################
# knowledge test
def knowledge_c_timeupdate(k):
    new = copy.deepcopy(k)
    new["knowledge"]["standard_question"] += f"_{time.strftime('%Y-%m-%d/%H:%M:%S',time.localtime())}"
    return new


def knowledge_u_timeupdate(knowledge_c):
    new = copy.deepcopy(knowledge_c)
    knowledge_u = new["knowledge"]
    knowledge_u["standard_question"] += f"-改_{time.strftime('%Y-%m-%d/%H:%M:%S',time.localtime())}"
    return knowledge_u


knowledge_c_1 = {
    "knowledge": {
        "status": True,
        "standard_question": "sdk测试什么才是标准问",
        "respond_all": True,
        "maintained_by_user_attribute_group": True
    },
    "knowledge_tag_id": ""
}
knowledge_c_2 = {
    "knowledge": {
        "status": True,
        "standard_question": "sdk测试如何退换货",
        "respond_all": True,
        "maintained_by_user_attribute_group": True,
        "id": None
    },
    "knowledge_tag_id": ""
}


# normal
# knowledge_tags / knowledge_items / create_knowledge / update_knowledge / similar-questions
@pytest.mark.parametrize('page,page_size,knowledge_c,knowledge_u', [
    (1, 50, knowledge_c_timeupdate(knowledge_c_1), knowledge_u_timeupdate(knowledge_c_1)),
    (1, 50, knowledge_c_timeupdate(knowledge_c_2), knowledge_u_timeupdate(knowledge_c_2)),
])
def test_knowledge_tags(page, page_size, knowledge_c, knowledge_u):
    client = WulaiClient(pubkey, secret, debug=True)
    # tags
    #    -- without parent_k_tag_id
    resp_parent_tags = client.knowledge_tags(page, page_size)
    parent_k_tag_id = resp_parent_tags.knowledge_tags[0].id

    #    -- with parent_k_tag_id
    resp_second_tags = client.knowledge_tags(page, page_size, parent_k_tag_id)
    knowledge_tags = resp_second_tags.knowledge_tags
    knowledge_tags_di = [r.to_dict() for r in knowledge_tags]
    target_id = find_value_from_dict_array(knowledge_tags_di, "name", "id", "测试_yt")

    # create
    knowledge_c["knowledge_tag_id"] = target_id
    resp_knowledge_c = client.create_knowledge(knowledge_c)
    knowledge_id = resp_knowledge_c.knowledge_tag_knowledge.knowledge.id

    # update
    knowledge_u["id"] = knowledge_id
    resp_knowledge_u = client.update_knowledge(knowledge_u)

    # items
    resp = client.knowledge_items(page, page_size=10)

    assert resp_knowledge_u.knowledge.id == knowledge_id
    assert resp.page_count

    # similar-question
    # create
    similar_question_c = {
        "knowledge_id": knowledge_id,
        "question": "我能在你左边画条龙嘛？"
    }
    resp_similar_c = client.create_similar_question(similar_question_c)

    # update
    similar_question_id = resp_similar_c.similar_question.id
    similar_question_u = {
        "knowledge_id": knowledge_id,
        "question": "我能在你左边画条龙嘛？-改",
        "id": similar_question_id
    }
    resp_similar_u = client.update_similar_question(similar_question_u)

    # list
    resp_similar_list = client.similar_questions(page, page_size, knowledge_id, timeout=10)
    assert resp_similar_u.similar_question.to_dict() in [r.to_dict() for r in resp_similar_list.similar_questions]

    # delete
    resp = client.delete_similar_question(similar_question_id)
    assert isinstance(resp, dict)


# user-attribute test
# 区分3.6和3.7的测试内容
if sys.version_info.minor == 7:
    source_name = "微信"
else:
    source_name = "微博"
user_attribute_group_item_1 = {
    "user_attribute_user_attribute_value": [
        {
            "user_attribute": {"id": "101627"},
            "user_attribute_value": {"name": source_name}
        },
        {
            "user_attribute": {"id": "101629"},
            "user_attribute_value": {"name": "新用户"}
        }
    ],
    "user_attribute_group": {"name": f"{source_name}新用户"}
}
user_attribute_group_item_update_1 = {
    "user_attribute_user_attribute_value": [
        {
            "user_attribute": {"id": "101627"},
            "user_attribute_value": {"name": source_name}
        },
        {
            "user_attribute": {"id": "101629"},
            "user_attribute_value": {"name": "老用户"}
        }
    ],
    "user_attribute_group": {
        "id": "",
        "name": f"{source_name}老用户"
    }
}
user_attribute_group_answer_1 = {
    "answer": {
        "knowledge_id": "1048247",
        "msg_body": {"text": {"content": f"这是测试属性组的答案--{source_name}老用户"}},
    },
    "user_attribute_group_id": ""
}
user_attribute_group_answer_update_1 = {
    "answer": {
        "knowledge_id": "1048247",
        "msg_body": {"text": {"content": f"这是测试属性组的答案--{source_name}老用户"}},
        "id": ""
    },
    "user_attribute_group_id": ""
}


@pytest.mark.parametrize('user_attribute_group_item,user_attribute_group_item_update,user_attribute_group_answer,'
                         'user_attribute_group_answer_update,page,page_size', [
    (user_attribute_group_item_1, user_attribute_group_item_update_1, user_attribute_group_answer_1,
     user_attribute_group_answer_update_1, 1, 50),
])
def test_user_attribute(user_attribute_group_item, user_attribute_group_item_update, user_attribute_group_answer,
                        user_attribute_group_answer_update, page, page_size):
    # 属性组测试知识点分类id：125537，知识点id： 1048247
    client = WulaiClient(pubkey, secret, debug=True)
    # 创建属性组
    # todo：每一次测试后需要去项目里手动删除这个属性组
    resp_cuagi = client.create_user_attribute_group_items(user_attribute_group_item)
    # 更新属性组
    #   属性组id
    user_attribute_group_id = resp_cuagi.user_attribute_group_item.user_attribute_group.id

    user_attribute_group_item_update["user_attribute_group"]["id"] = user_attribute_group_id

    resp_uuagi = client.update_user_attribute_group_items(user_attribute_group_item_update)

    #   属性组名称
    user_attribute_group_name = resp_uuagi.user_attribute_group_item.user_attribute_group.name

    # 查询属性组及属性列表
    resp_uagi = client.user_attribute_group_items(page, page_size, timeout=10)
    user_attribute_group_ids = [uagi.user_attribute_group.id for uagi in resp_uagi.user_attribute_group_items]
    user_attribute_group_names = [uagi.user_attribute_group.name for uagi in resp_uagi.user_attribute_group_items]
    assert user_attribute_group_id in user_attribute_group_ids
    assert user_attribute_group_name in user_attribute_group_names

    # 创建属性组回复
    user_attribute_group_answer["user_attribute_group_id"] = user_attribute_group_id

    resp_cuaga = client.create_user_attribute_group_answer(user_attribute_group_answer)
    #   属性组答案id
    user_attribute_group_answer_id = resp_cuaga.user_attribute_group_answer.answer.id
    # 更新属性组回复
    user_attribute_group_answer_update["answer"]["id"] = user_attribute_group_answer_id
    user_attribute_group_answer_update["user_attribute_group_id"] = user_attribute_group_id

    resp_uuaga = client.update_user_attribute_group_answer(user_attribute_group_answer_update)
    assert resp_uuaga.user_attribute_group_answer.answer.id == user_attribute_group_answer_id
    # 查询属性组回复列表
    resp_answers = client.user_attribute_group_answers(page, page_size, timeout=10)
    resp_answers_2 = client.user_attribute_group_answers(page, page_size, {"knowledge_id": "1048247"}, timeout=10)
    assert isinstance(resp_answers.user_attribute_group_answers, list)
    assert isinstance(resp_answers_2.user_attribute_group_answers, list)

    # 删除属性组回复
    resp = client.delete_user_attribute_group_answer(user_attribute_group_answer_id)


#############################################################
#                   reset task bot
#############################################################
@pytest.mark.parametrize('user_id,msg_body,extra', [
    ("shierlou", {"text": {"content": "/restart"}}, "this is extra string")
])
def test_reset_task(user_id, msg_body, extra):
    client = WulaiClient(pubkey, secret, debug=True)
    resp = client.get_bot_response(user_id, msg_body, extra)
    assert isinstance(resp.suggested_response[0].response[0].msg_body.text.content, str)



import os
import pytest

from wulaisdk.client import WulaiClient


pubkey = os.getenv("PUBKEY", "")
secret = os.getenv("SECRET", "")
log_dir_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


#############################################################
#                       自然语言处理类
#############################################################
# 实体抽取测试
@pytest.mark.parametrize('query', [
    '灵刻车型有悦翔V5，江南TT和逸动',
    '我的ip地址是180.167.137.10',
])
def test_entities_extract(query):
    client = WulaiClient(pubkey, secret, debug=True)

    resp = client.entities_extract(query)
    assert resp.to_dict()
    for entity in resp.entities:
        if hasattr(entity.entity, "system_entity"):
            assert entity.entity.system_entity.standard_value
        elif hasattr(entity.entity, "enumeration_entity"):
            assert entity.entity.enumeration_entity.standard_value
        elif hasattr(entity.entity, "regex_entity"):
            assert entity.entity.regex_entity.name
        else:
            # error
            assert False


# 分词&词性标注
@pytest.mark.parametrize('query', [
    '灵刻车型有悦翔V5，江南TT和逸动',
    '我的ip地址是180.167.137.10',
])
def test_tokenize(query):
    client = WulaiClient(pubkey, secret, debug=True)

    resp = client.tokenize(query)
    assert resp.tokens[0].text



import six
import inspect


class BaseModel:

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.__dict__):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    @classmethod
    def from_dict(cls, env):
        if not isinstance(env, dict):
            return env
        parameters = inspect.signature(cls).parameters
        return cls(**{
            k: v for k, v in env.items()
            if k in parameters or set(parameters.keys()) == {'kwargs'}
        })



"""
统计类
"""

from typing import List
from wulaisdk.response import BaseModel


class SatisfactionStats(BaseModel):
    """
    满意度
    """
    thumb_up_count: int
    wrong_answer_count: int
    bad_answer_count: int

    def __init__(self, thumb_up_count: int, wrong_answer_count: int, bad_answer_count: int) -> None:
        self.thumb_up_count = thumb_up_count
        self.wrong_answer_count = wrong_answer_count
        self.bad_answer_count = bad_answer_count


class QASatisfactionKnowledgeStat(BaseModel):
    """
    知识点粒度的问答满意度列表
    """
    knowledge_id: int
    satisfaction_stats: SatisfactionStats
    standard_question: str

    def __init__(self, knowledge_id: int, satisfaction_stats: SatisfactionStats, standard_question: str) -> None:
        self.knowledge_id = knowledge_id
        self.satisfaction_stats = SatisfactionStats.from_dict(satisfaction_stats)
        self.standard_question = standard_question


class StatsQASatisfactionKnowledgeDaily(BaseModel):
    """
    查询问答满意度评价统计列表（知识点粒度，日报）
    """
    page_count: int
    qa_satisfaction_knowledge_stats: List[QASatisfactionKnowledgeStat]

    def __init__(self, page_count: int, qa_satisfaction_knowledge_stats: List[QASatisfactionKnowledgeStat]) -> None:
        self.page_count = page_count
        self.qa_satisfaction_knowledge_stats = [
            QASatisfactionKnowledgeStat.from_dict(qsks) for qsks in qa_satisfaction_knowledge_stats
        ]


class MessageStats(BaseModel):
    """
    消息统计
    """
    receive_count: int

    def __init__(self, receive_count: int) -> None:
        self.receive_count = receive_count


class QARecallStats(BaseModel):
    """
    问答召回数
    """
    recall_count: int

    def __init__(self, recall_count: int) -> None:
        self.recall_count = recall_count


class QARecallDailyStat(BaseModel):
    """
    (天粒度的的问答召回数列表)
    """
    date: str
    message_stats: MessageStats
    qa_recall_stats: QARecallStats

    def __init__(self, date: str, message_stats: MessageStats, qa_recall_stats: QARecallStats) -> None:
        self.date = date
        self.message_stats = MessageStats.from_dict(message_stats)
        self.qa_recall_stats = QARecallStats.from_dict(qa_recall_stats)


class StatsQARecallDaily(BaseModel):
    """
    查询问答召回数统计列表（日报）
    """
    qa_recall_daily_stats: List[QARecallDailyStat]

    def __init__(self, qa_recall_daily_stats: List[QARecallDailyStat]) -> None:
        self.qa_recall_daily_stats = [QARecallDailyStat.from_dict(qrds) for qrds in qa_recall_daily_stats]


class QARecallKnowledgeStat(BaseModel):
    """
    知识点粒度的问答满意度列表
    """
    knowledge_id: int
    standard_question: str
    qa_recall_stats: QARecallStats

    def __init__(self, knowledge_id: int, standard_question: str, qa_recall_stats: QARecallStats) -> None:
        self.knowledge_id = knowledge_id
        self.standard_question = standard_question
        self.qa_recall_stats = QARecallStats.from_dict(qa_recall_stats)


class StatasQARecallDailyKnowledges(BaseModel):
    """
    查询问答召回数统计列表（知识点粒度，日报）
    """
    qa_recall_knowledge_stats: List[QARecallKnowledgeStat]
    page_count: int

    def __init__(self, qa_recall_knowledge_stats: List[QARecallKnowledgeStat], page_count: int) -> None:
        self.qa_recall_knowledge_stats = [QARecallKnowledgeStat.from_dict(qcks) for qcks in qa_recall_knowledge_stats]
        self.page_count = page_count



"""
自然语言处理类
"""
from wulaisdk.response import BaseModel
from typing import List


class EnumerationEntity(BaseModel):
    """
    枚举实体
    """
    text: str
    synonyms: List[str]
    standard_value: str
    name: str

    def __init__(self, text: str, synonyms: List[str], standard_value: str, name: str) -> None:
        self.text = text
        self.synonyms = synonyms
        self.standard_value = standard_value
        self.name = name


class RegexEntity(BaseModel):
    """
    正则实体
    """
    text: str
    name: str

    def __init__(self, text: str, name: str) -> None:
        self.text = text
        self.name = name


class SystemEntity(BaseModel):
    """
    系统实体
    """
    text: str
    standard_value: str
    name: str

    def __init__(self, text: str, standard_value: str, name: str) -> None:
        self.text = text
        self.standard_value = standard_value
        self.name = name


class Entity(BaseModel):
    entity: dict

    def __init__(self, **kwargs) -> None:
        if "enumeration_entity" in kwargs:
            self.enumeration_entity = EnumerationEntity.from_dict(kwargs.get("enumeration_entity"))
        elif "regex_entity" in kwargs:
            self.regex_entity = RegexEntity.from_dict(kwargs.get("regex_entity"))
        elif "system_entity" in kwargs:
            self.system_entity = SystemEntity.from_dict(kwargs.get("system_entity"))
        else:
            for k, v in kwargs.items():
                setattr(self, k, v)


class EntityElement(BaseModel):
    """
    实体抽取的返回值
    """
    type: str
    idx_start: int
    entity: Entity

    def __init__(self, type: str, idx_start: int, entity: Entity) -> None:
        self.type = type
        self.idx_start = idx_start
        self.entity = Entity.from_dict(entity)


class EntityExtract(BaseModel):
    """
    实体抽取
    """
    entities: List[EntityElement]

    def __init__(self, entities: List[EntityElement]) -> None:
        self.entities = [EntityElement.from_dict(entity) for entity in entities]


class Token(BaseModel):
    """
    文本&词性
    """
    text: str
    pos: str
    idx_start: int

    def __init__(self, text: str, pos: str, idx_start: int) -> None:
        self.text = text
        self.pos = pos
        self.idx_start = idx_start


class Tokenize(BaseModel):
    """
    分词&词性标注
    """
    tokens: List[Token]

    def __init__(self, tokens: List[Token]) -> None:
        self.tokens = [Token.from_dict(token) for token in tokens]



"""
任务类
"""
from typing import List
from wulaisdk.response import BaseModel
from wulaisdk.response.msg_body import MsgBody


class Scene(BaseModel):
    """
    场景
    """
    description: str
    intent_switch_mode: str
    id: int
    smart_slot_filling_threshold: int
    name: str

    def __init__(self, description: str, intent_switch_mode: str, id: int,
                 smart_slot_filling_threshold: int, name: str) -> None:
        self.description = description
        self.intent_switch_mode = intent_switch_mode
        self.id = id
        self.smart_slot_filling_threshold = smart_slot_filling_threshold
        self.name = name


class Intent(BaseModel):
    """
    意图
    """
    scene_id: int
    status: bool
    lifespan_mins: int
    id: int
    name: str

    def __init__(self, scene_id: int, status: bool, lifespan_mins: int, id: int, name: str) -> None:
        self.scene_id = scene_id
        self.status = status
        self.lifespan_mins = lifespan_mins
        self.id = id
        self.name = name


class IntentTrigger(BaseModel):
    """
    触发器
    """
    text: str
    intent_id: int
    type: str
    id: int

    def __init__(self, text: str, intent_id: int, type: str, id: int) -> None:
        self.text = text
        self.intent_id = intent_id
        self.type = type
        self.id = id


class SlotSimple(BaseModel):
    """
    词槽(简版)
    """
    id: int
    name: str

    def __init__(self, id: int, name: str) -> None:
        self.id = id
        self.name = name


class Slot(BaseModel):
    """
    词槽
    """
    scene_id: int
    query_slot_filling: bool
    id: int
    name: str

    def __init__(self, scene_id: int, query_slot_filling: bool, id: int, name: str) -> None:
        self.scene_id = scene_id
        self.query_slot_filling = query_slot_filling
        self.id = id
        self.name = name


class DataSource(BaseModel):
    """
    词槽数据来源
    """
    entity_id: int
    slot_id: int
    id: int

    def __init__(self, entity_id: int, slot_id: int, id: int) -> None:
        self.entity_id = entity_id
        self.slot_id = slot_id
        self.id = id


class RecommendIntent(BaseModel):
    """
    推荐意图
    """
    intent_id: int
    score: int
    intent_name: str

    def __init__(self, intent_id: int, score: int, intent_name: str) -> None:
        self.intent_id = intent_id
        self.score = score
        self.intent_name = intent_name


class QueryItem(BaseModel):
    """
    待审核问题
    """
    content: str
    id: int
    recommend_intent: RecommendIntent

    def __init__(self, content: str, id: int, recommend_intent: RecommendIntent) -> None:
        self.content = content
        self.id = id
        self.recommend_intent = RecommendIntent.from_dict(recommend_intent)


# api start
class Scenes(BaseModel):
    """
    查询场景列表
    """
    scenes: List[Scene]

    def __init__(self, scenes: List[Scene]) -> None:
        self.scenes = [Scene.from_dict(scene) for scene in scenes]


class CreateScene(BaseModel):
    """
    创建场景
    """
    scene: Scene

    def __init__(self, scene: Scene) -> None:
        self.scene = Scene.from_dict(scene)


class UpdateScene(BaseModel):
    """
    更新场景
    """
    scene: Scene

    def __init__(self, scene: Scene) -> None:
        self.scene = Scene.from_dict(scene)


class Intents(BaseModel):
    """
    查询意图列表
    """
    intents: List[Intent]

    def __init__(self, intents: List[Intent]) -> None:
        self.intents = [Intent.from_dict(intent) for intent in intents]


class CreateIntent(BaseModel):
    """
    创建意图
    """
    intent: Intent

    def __init__(self, intent: Intent) -> None:
        self.intent = Intent.from_dict(intent)


class UpdateIntent(BaseModel):
    """
    更新意图
    """
    intent: Intent

    def __init__(self, intent: Intent) -> None:
        self.intent = Intent.from_dict(intent)


class IntentTriggers(BaseModel):
    """
    触发器列表
    """
    intent_triggers: List[IntentTrigger]

    def __init__(self, intent_triggers: List[IntentTrigger]) -> None:
        self.intent_triggers = [IntentTrigger.from_dict(intent_trigger) for intent_trigger in intent_triggers]


class CreateIntentTrigger(BaseModel):
    """
    创建触发器
    """
    intent_trigger: IntentTrigger

    def __init__(self, intent_trigger: IntentTrigger) -> None:
        self.intent_trigger = IntentTrigger.from_dict(intent_trigger)


class UpdateIntentTrigger(BaseModel):
    """
    更新触发器
    """
    intent_trigger: IntentTrigger

    def __init__(self, intent_trigger: IntentTrigger) -> None:
        self.intent_trigger = IntentTrigger.from_dict(intent_trigger)


class Slots(BaseModel):
    """
    词槽列表
    """
    slots: List[SlotSimple]

    def __init__(self, slots: List[SlotSimple]) -> None:
        self.slots = [SlotSimple.from_dict(slot) for slot in slots]


class CreateSlot(BaseModel):
    """
    创建词槽
    """
    slot: Slot

    def __init__(self, slot: Slot) -> None:
        self.slot = Slot.from_dict(slot)


class UpdateSlot(BaseModel):
    """
    更新词槽
    """
    slot: Slot

    def __init__(self, slot: Slot) -> None:
        self.slot = Slot.from_dict(slot)


class GetSlot(BaseModel):
    """
    词槽详情
    """
    slot: Slot

    def __init__(self, slot: Slot) -> None:
        self.slot = Slot.from_dict(slot)


class SlotDataSource(BaseModel):
    """
    查询词槽数据来源
    """
    data_sources: List[DataSource]

    def __init__(self, data_sources: List[DataSource]) -> None:
        self.data_sources = [DataSource.from_dict(data_source) for data_source in data_sources]


class CreateSlotDataSource(BaseModel):
    """
    创建词槽数据来源
    """
    data_source: DataSource

    def __init__(self, data_source: DataSource) -> None:
        self.data_source = DataSource.from_dict(data_source)


class Response(BaseModel):
    """
    单元内回复
    """
    response: MsgBody
    id: int
    block_id: int

    def __init__(self, response: MsgBody, id: int, block_id: int) -> None:
        self.response = MsgBody.from_dict(response)
        self.id = id
        self.block_id = block_id


class Last(BaseModel):
    """
    意图终点跳转上个意图
    """

    def __init__(self, **kwargs) -> None:
        for k, v in kwargs.items():
            self.k = v


class End(BaseModel):
    """
    意图终点单元跳转指定意图
    """

    def __init__(self, **kwargs) -> None:
        for k, v in kwargs.items():
            self.k = v


class Specified(BaseModel):
    """
    意图终点单元跳转指定意图
    """
    id: int

    def __init__(self, id: int) -> None:
        self.id = id


class Action(BaseModel):
    """
    结束单元跳转方式 (指定意图 / 上个意图 / 不跳转)
    """

    def __init__(self, **kwargs) -> None:
        if "last" in kwargs:
            self.last = Last.from_dict(kwargs.get("last"))
        elif "end" in kwargs:
            self.end = End.from_dict(kwargs.get("end"))
        elif "specified" in kwargs:
            self.specified = Specified.from_dict(kwargs.get("specified"))
        else:
            for k, v in kwargs.items():
                setattr(self, k, v)


class EndBlock(BaseModel):
    """
    意图终点单元
    """
    action: Action
    slot_memorizing: bool
    intent_id: int
    id: int
    name: str

    def __init__(self, action: Action, slot_memorizing: bool, intent_id: int, id: int, name: str) -> None:
        self.action = Action.from_dict(action)
        self.slot_memorizing = slot_memorizing
        self.intent_id = intent_id
        self.id = id
        self.name = name


class Block(BaseModel):
    """
    单元
    """
    type: str
    id: int
    name: str

    def __init__(self, type: str, id: int, name: str) -> None:
        self.type = type
        self.id = id
        self.name = name


# 单元
class Default(BaseModel):
    """
    单元跳转条件 默认
    """

    def __init__(self, **kwargs) -> None:
        for k, v in kwargs.items():
            self.k = v


class InEntity(BaseModel):
    """
    单元跳转条件 属于(实体)
    """
    id: int

    def __init__(self, id: int) -> None:
        self.id = id


class NotInEntity(BaseModel):
    """
    单元跳转条件 不属于(实体)
    """
    id: int

    def __init__(self, id: int) -> None:
        self.id = id


class EqualTo(BaseModel):
    """
    单元跳转条件 等于
    """
    value: str

    def __init__(self, value: str) -> None:
        self.value = value


class NotEqualTo(BaseModel):
    """
    单元跳转条件 不等于
    """
    value: str

    def __init__(self, value: str) -> None:
        self.value = value


class LessThan(BaseModel):
    """
    单元跳转条件 小于
    """
    value: float

    def __init__(self, value: float) -> None:
        self.value = value


class LessThanOrEqualTo(BaseModel):
    """
    单元跳转条件 小于等于
    """
    value: float

    def __init__(self, value: float) -> None:
        self.value = value


class GreaterThan(BaseModel):
    """
    单元跳转条件 大于
    """
    value: float

    def __init__(self, value: float) -> None:
        self.value = value


class GreaterThanOrEqualTo(BaseModel):
    """
    单元跳转条件 大于等于
    """
    value: float

    def __init__(self, value: float) -> None:
        self.value = value


class MatchRegex(BaseModel):
    """
    单元跳转条件 符合正则
    """
    regex: str

    def __init__(self, regex: str) -> None:
        self.regex = regex


class DismatchRegex(BaseModel):
    """
    单元跳转条件 不符合正则
    """
    regex: str

    def __init__(self, regex: str) -> None:
        self.regex = regex


class Include(BaseModel):
    """
    单元跳转条件 包含
    """
    value: str

    def __init__(self, value: str) -> None:
        self.value = value


class Exclude(BaseModel):
    """
    单元跳转条件 不包含
    """
    value: str

    def __init__(self, value: str) -> None:
        self.value = value


class Condition(BaseModel):
    """
    单元跳转条件(默认 / 大于 / 大于等于 / 小于 / 小于等于 / 等于 / 不等于 / 包含 / 不包含 / 属于实体 / 不属于实体 / 符合正则 / 不符合正则)
    """

    def __init__(self, **kwargs) -> None:
        if "default" in kwargs:
            self.default = Default.from_dict(kwargs.get("default"))
        elif "in_entity" in kwargs:
            self.in_entity = InEntity.from_dict(kwargs.get("in_entity"))
        elif "not_in_entity" in kwargs:
            self.not_in_entity = NotInEntity.from_dict(kwargs.get("not_in_entity"))
        elif "equal_to" in kwargs:
            self.equal_to = EqualTo.from_dict(kwargs.get("equal_to"))
        elif "not_equal_to" in kwargs:
            self.not_equal_to = NotEqualTo.from_dict(kwargs.get("not_equal_to"))
        elif "greater_than" in kwargs:
            self.greater_than = GreaterThan.from_dict(kwargs.get("greater_than"))
        elif "greater_than_or_equal_to" in kwargs:
            self.greater_than_or_equal_to = GreaterThanOrEqualTo.from_dict(kwargs.get("greater_than_or_equal_to"))
        elif "less_than" in kwargs:
            self.less_than = LessThan.from_dict(kwargs.get("less_than"))
        elif "less_than_or_equal_to" in kwargs:
            self.less_than_or_equal_to = LessThanOrEqualTo.from_dict(kwargs.get("less_than_or_equal_to"))
        elif "match_regex" in kwargs:
            self.match_regex = MatchRegex.from_dict(kwargs.get("match_regex"))
        elif "match_regex" in kwargs:
            self.match_regex = MatchRegex.from_dict(kwargs.get("match_regex"))
        elif "dismatch_regex" in kwargs:
            self.dismatch_regex = DismatchRegex.from_dict(kwargs.get("dismatch_regex"))
        elif "include" in kwargs:
            self.include = Include.from_dict(kwargs.get("include"))
        elif "exclude" in kwargs:
            self.exclude = Exclude.from_dict(kwargs.get("exclude"))
        else:
            for k, v in kwargs.items():
                setattr(self, k, v)


class Connection(BaseModel):
    """
    单元关系
    """
    from_block_id: int
    to_block_id: int
    condition: Condition

    def __init__(self, from_block_id: int, to_block_id: int, condition: Condition) -> None:
        self.from_block_id = from_block_id
        self.to_block_id = to_block_id
        self.condition = Condition.from_dict(condition)


class Relation(BaseModel):
    """
    单元关系
    """
    connection: Connection
    intent_id: int
    id: int

    def __init__(self, connection: Connection, intent_id: int, id: int) -> None:
        self.connection = Connection.from_dict(connection)
        self.intent_id = intent_id
        self.id = id


class InformBlock(BaseModel):
    """
    消息发送单元
    """
    responses: List[MsgBody]
    connection: Connection
    mode: str
    intent_id: int
    id: int
    name: str

    def __init__(self, responses: List[MsgBody], connection: Connection, mode: str, intent_id: int, id: int, name: str) -> None:
        self.responses = [MsgBody.from_dict(response) for response in responses]
        self.connection = Connection.from_dict(connection)
        self.mode = mode
        self.intent_id = intent_id
        self.id = id
        self.name = name


class RequestBlock(BaseModel):
    """
    询问填槽单元
    """
    name: str
    default_slot_value: str
    slot_filling_when_asked: bool
    connections: List[Connection]
    slot_id: int
    mode: str
    request_count: int
    intent_id: int
    id: int
    responses: List[MsgBody]

    def __init__(
            self, name: str, default_slot_value: str, slot_filling_when_asked: bool,
            connections: List[Connection], slot_id: int, mode: str, request_count: int,
            intent_id: int, id: int, responses: List[MsgBody]) -> None:
        self.name = name
        self.default_slot_value = default_slot_value
        self.slot_filling_when_asked = slot_filling_when_asked
        self.connections = [Connection.from_dict(connection) for connection in connections]
        self.slot_id = slot_id
        self.mode = mode
        self.request_count = request_count
        self.intent_id = intent_id
        self.id = id
        self.responses = [MsgBody.from_dict(response) for response in responses]


class GetInformBlock(BaseModel):
    """
    查询消息发送单元
    """
    block: InformBlock

    def __init__(self, block: InformBlock) -> None:
        self.block = InformBlock.from_dict(block)


class CreateInformBlock(BaseModel):
    """
    创建消息发送单元
    """
    block: InformBlock

    def __init__(self, block: InformBlock) -> None:
        self.block = InformBlock.from_dict(block)


class UpdateInformBlock(BaseModel):
    """
    更新消息发送单元
    """
    block: InformBlock

    def __init__(self, block: InformBlock) -> None:
        self.block = InformBlock.from_dict(block)


class GetRequestBlock(BaseModel):
    """
    查询询问填槽单元
    """
    block: RequestBlock

    def __init__(self, block: RequestBlock) -> None:
        self.block = RequestBlock.from_dict(block)


class CreateRequestBlock(BaseModel):
    """
    创建询问填槽单元
    """
    block: RequestBlock

    def __init__(self, block: RequestBlock) -> None:
        self.block = RequestBlock.from_dict(block)


class UpdateRequestBlock(BaseModel):
    """
    更新询问填槽单元
    """
    block: RequestBlock

    def __init__(self, block: RequestBlock) -> None:
        self.block = RequestBlock.from_dict(block)


class CreateResponse(BaseModel):
    """
    创建单元内回复
    """
    response: Response

    def __init__(self, response: Response) -> None:
        self.response = Response.from_dict(response)


class UpdateResponse(BaseModel):
    """
    更新单元内回复
    """
    response: Response

    def __init__(self, response: Response) -> None:
        self.response = Response.from_dict(response)


class GetEndBlock(BaseModel):
    """
    查询意图终点单元
    """
    block: EndBlock

    def __init__(self, block: EndBlock) -> None:
        self.block = EndBlock.from_dict(block)


class CreateEndBlock(BaseModel):
    """
    创建意图终点单元
    """
    block: EndBlock

    def __init__(self, block: EndBlock) -> None:
        self.block = EndBlock.from_dict(block)


class UpdateEndBlock(BaseModel):
    """
    更新意图终点单元
    """
    block: EndBlock

    def __init__(self, block: EndBlock) -> None:
        self.block = EndBlock.from_dict(block)


class Blocks(BaseModel):
    """
    查询单元列表
    """
    blocks: List[Block]

    def __init__(self, blocks: List[Block]) -> None:
        self.blocks = [Block.from_dict(block) for block in blocks]


class CreateBlockRelation(BaseModel):
    """
    创建单元关系
    """
    relation: Relation

    def __init__(self, relation: Relation) -> None:
        self.relation = Relation.from_dict(relation)


class IntentTriggerLearning(BaseModel):
    """
    待审核问题列表
    """
    query_items: List[QueryItem]

    def __init__(self, query_items: List[QueryItem]) -> None:
        self.query_items = [QueryItem.from_dict(query_item) for query_item in query_items]


class UpdateIntentStatus(BaseModel):
    """
    更新意图状态
    """
    status: bool
    first_block_id: int
    intent_id: int
    update_time: str

    def __init__(self, status: bool, first_block_id: int, intent_id: int, update_time: str) -> None:
        self.status = status
        self.first_block_id = first_block_id
        self.intent_id = intent_id
        self.update_time = update_time



#!/usr/bin/env python
# -*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: shierlou
# Mail: manbugv@gmail.com
# Created Time: 2019-08-27 10:27:00
#############################################


from setuptools import setup, find_packages

setup(
    name="wulaisdk",
    version="1.1.5",
    description="Laiye Wulai SDK for Python Programming Language",
    long_description="Laiye Wulai SDK for Python Programming Language",
    license="LICENSE Apache 2.0",

    url="https://github.com/laiye-ai/wulai-openapi-sdk-python",
    author="shierlou",
    author_email="manbugv@gmail.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=["requests"]
)



"""
词库管理类
"""

from typing import List
from wulaisdk.response import BaseModel


class Entity(BaseModel):
    """
    实体
    """
    type: str
    id: int
    name: str

    def __init__(self, type: str, id: int, name: str) -> None:
        self.type = type
        self.id = id
        self.name = name


class DictionaryEntities(BaseModel):
    """
    全部实体概要列表
    """
    entities: List[Entity]

    def __init__(self, entities: List[Entity]) -> None:
        self.entities = [Entity.from_dict(entity) for entity in entities]


class Term(BaseModel):
    """
    专有词汇详情
    """
    id: int
    name: str

    def __init__(self, id: int, name: str) -> None:
        self.id = id
        self.name = name


class TermItem(BaseModel):
    """
    专有词汇
    """
    term: Term
    synonyms: List[str]

    def __init__(self, term: Term, synonyms: List[str]) -> None:
        self.term = Term.from_dict(term)
        self.synonyms = synonyms


class DictionaryTerms(BaseModel):
    """
    专有词汇列表
    """
    term_item: List[TermItem]
    page_count: int

    def __init__(self, term_item: List[TermItem], page_count: int) -> None:
        self.term_item = [TermItem.from_dict(ti) for ti in term_item]
        self.page_count = page_count


class DictionaryTerm(BaseModel):
    """
    创建/更新专有词汇
    """
    term_item: TermItem

    def __init__(self, term_item: TermItem) -> None:
        self.term_item = TermItem.from_dict(term_item)


class Value(BaseModel):
    """
    实体值
    """
    synonyms: List[str]
    standard_value: str

    def __init__(self, synonyms: List[str], standard_value: str) -> None:
        self.synonyms = synonyms
        self.standard_value = standard_value


class IntentEntityValue(Value):
    """
    意图实体值
    """
    pass


class SystemEntityValue(BaseModel):
    """
    预设实体
    """
    description: str

    def __init__(self, description: str) -> None:
        self.description = description


class RegexEntityValue(BaseModel):
    """
    正则实体
    """
    regex: str

    def __init__(self, regex: str) -> None:
        self.regex = regex


class EnumEntityValue(BaseModel):
    """
    枚举实体值
    """
    values: List[Value]

    def __init__(self, values: List[Value]) -> None:
        self.values = [Value.from_dict(value) for value in values]


class EntityValue(BaseModel):
    """
    实体值
    """
    value: dict

    def __init__(self, **kwargs) -> None:
        if "intent_entity_value" in kwargs:
            self.text = IntentEntityValue.from_dict(kwargs.get("intent_entity_value"))
        elif "system_entity_value" in kwargs:
            self.image = SystemEntityValue.from_dict(kwargs.get("system_entity_value"))
        elif "regex_entity_value" in kwargs:
            self.custom = RegexEntityValue.from_dict(kwargs.get("regex_entity_value"))
        elif "enumeration_entity_value" in kwargs:
            self.video = EnumEntityValue.from_dict(kwargs.get("enumeration_entity_value"))
        else:
            for k, v in kwargs.items():
                setattr(self, k, v)


class EntityDetail(BaseModel):
    """
    实体详情
    """
    type: str
    id: int
    value: EntityValue
    name: str

    def __init__(self, type: str, id: int, value: Value, name: str) -> None:
        self.type = type
        self.id = id
        self.value = EntityValue.from_dict(value)
        self.name = name


class DictionaryEntity(BaseModel):
    """
    查询一个实体详情
    """
    entity: EntityDetail

    def __init__(self, entity: EntityDetail) -> None:
        self.entity = EntityDetail.from_dict(entity)


class EnumEntity(BaseModel):
    """
    枚举实体详情
    """
    values: List[Value]
    id: int
    name: str

    def __init__(self, values: List[Value], id: int, name: str) -> None:
        self.values = [Value.from_dict(value) for value in values]
        self.id = id
        self.name = name


class CreateEnumEntity(BaseModel):
    """
    创建枚举实体
    """
    enum_entity: EnumEntity

    def __init__(self, enum_entity: EnumEntity) -> None:
        self.enum_entity = EnumEntity.from_dict(enum_entity)


class CreateEnumEntityValue(BaseModel):
    """
    创建枚举实体值
    """
    enum_entity: EnumEntity

    def __init__(self, enum_entity: EnumEntity) -> None:
        self.enum_entity = EnumEntity.from_dict(enum_entity)


class IntentEntity(BaseModel):
    """
    意图实体
    """
    id: int
    value: Value
    name: str

    def __init__(self, id: int, value: Value, name: str) -> None:
        self.id = id
        self.value = Value.from_dict(value)
        self.name = name


class CreateIntentEntity(BaseModel):
    """
    创建意图实体
    """
    intent_entity: IntentEntity

    def __init__(self, intent_entity: IntentEntity) -> None:
        self.intent_entity = IntentEntity.from_dict(intent_entity)


class CreateIntentEntityValue(BaseModel):
    """
    创建意图实体值相似说法
    """
    intent_entity: IntentEntity

    def __init__(self, intent_entity: IntentEntity) -> None:
        self.intent_entity = IntentEntity.from_dict(intent_entity)



from typing import List

from wulaisdk.response import BaseModel
from wulaisdk.response.bot import Bot, Response


class SuggestedResponse(BaseModel):
    is_send: bool
    bot: Bot
    source: str
    score: int
    response: List[Response]
    quick_reply: List[str]

    def __init__(self, is_send: bool, bot: Bot, source: str, score: int, response: List[Response], quick_reply: List[str]) -> None:
        self.is_send = is_send
        self.bot = Bot.from_dict(bot)
        self.source = source
        self.score = score
        self.response = [Response.from_dict(r) for r in response]
        self.quick_reply = quick_reply


class BotResponse(BaseModel):
    is_dispatch: bool
    suggested_response: List[SuggestedResponse]
    msg_id: str
    extra: str

    def __init__(self, is_dispatch: bool, suggested_response: List[SuggestedResponse], msg_id: str, extra: str) -> None:
        self.is_dispatch = is_dispatch
        self.suggested_response = [SuggestedResponse.from_dict(sr) for sr in suggested_response]
        self.msg_id = msg_id
        self.extra = extra
